#include <nwnet.h>
#include <stdio.h>

#define FALSE 0
#define TRUE 1

// ***************************
// Example code for Add Object
// ***************************
void main()
{ int               i,err;
  NWDSContextHandle nContext;
  Buf_T             *InpBuf;
  char              *attrName[]  = { "Object Class",
                         "Surname",
                         "Telephone Number"
                       };
  uint32            attrSyntax[] = { SYN_CLASS_NAME,
                         SYN_CI_STRING,
                         SYN_TEL_NUMBER
                       };
  char              *attrValue[] = { "User",
                         "Smith",
                         "429-0000"
                       };

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("Could not create context\n");
      exit(0);
    }

  // Set context as desired if necessary and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"O=Novell.C=US");
  if(err<0)
    { printf("Error in NWDSSetContext: <%d>\n",err);
      NWDSFreeContext(nContext);
      exit(0);
    }

  // Allocate space for input buffer and check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &InpBuf);
  if(err<0)
    { printf("Error in NWDSAllocBuf: <%d>\n",err);
      NWDSFreeContext(nContext);
      exit(0);
    }

  // Initialize input buffer and check for errors.  The output buffer does
  // not need to be initialized.   This buffer will be initialized to the
  // DSV_ADD_ENTRY operation.
  err = NWDSInitBuf(nContext, DSV_ADD_ENTRY, InpBuf);
  if(err<0)
    { printf("Error in NWDSInitBuf: <%d>\n",err);
      NWDSFreeContext(nContext);
      NWDSFreeBuf(InpBuf);
      exit(0);
    }

  // Put object attribute names, syntaxes for the object to be
  // added into the input buffer.
  for(i=0; i<3; i++)
    { // Put the name in the buffer.  Check for errors.
      err = NWDSPutAttrName(nContext,InpBuf,attrName[i]);
      if(err<0)
     { printf("Error in NWDSPutAttrName: <%d>\n",err);
       NWDSFreeContext(nContext);
       NWDSFreeBuf(InpBuf);
       exit(0);
     }
      // Put the value and syntax for attribute in the buffer.  Check for
      // errors.
      err = NWDSPutAttrVal(nContext, InpBuf, attrSyntax[i],attrValue[i]);
      if(err<0)
     { printf("Error in NWDSPutAttrVal: <%d>\n",err);
       NWDSFreeContext(nContext);
       NWDSFreeBuf(InpBuf);
       exit(0);
     }
    }

  // Call NWDSAddObject with object's context and name, the input buffer
  // already filled, and two other parms that are not yet documented.  Check
  // for errors.
  err = NWDSAddObject(nContext,"CN=SomeObject",NULL,0,InpBuf);
  if(err<0)
    { printf("Error in NWDSAddObject: <%d>\n",err);
      NWDSFreeContext(nContext);
      NWDSFreeBuf(InpBuf);
      exit(0);
    }

  // Clean up buffers and context.  These functions do not return anything.

  NWDSFreeContext(nContext);
  NWDSFreeBuf(InpBuf);
}
