//   ͻ
//                                                                       
//    module:      broker.c                                              
//    abstract:    This module contains the driver for the NDS version   
//                 of the BROKER Example NLM.                            
//                                                                       
//    environment: NetWare 4.0 ALPHA                                     
//                 Network C for NLMs SDK v4.0 ALPHA Update              
//                 CLib v4.00 ALPHA                                      
//                                                                       
//     This software is provided as is and carries no warranty           
//     whatsoever.  Novell disclaims and excludes any and all implied    
//     warranties of merchantability, title and fitness for a particular 
//     purpose.  Novell does not warrant that the software will satisfy  
//     your requirements or that the software is without defect or error 
//     or that operation of the software will be uninterrupted.  You are 
//     using the software at your risk.  The software is not a product   
//     of Novell, Inc. or any of subsidiaries.                           
//                                                                       
//   Ķ
//    maintenance history:                                               
//    level    date      pi   description                                
//   Ķ
//     001   06/26/92    md   initial release.                           
//   ͼ

#define NULL 0

#include <nwdsapi.h>
#include "newin400.h"
#include "broker.h"
#include <stdio.h>
#include <nwipxspx.h>
#include <nwsemaph.h>
#include <conio.h>
#include <process.h>
#include <time.h>
#include <nwenvrn.h>
#include <advanced.h>
#include <string.h>
#include <sap.h>
#include <nwdir.h>
#include <nwconn.h>
#include <signal.h>
#include <assert.h>

NWDSContextHandle context;

char admin[255];
char adminPassword[255];

CINFO **initClientInfoHead;
CINFO **unInitClientInfoHead;
unsigned long clientListSem, initializeSem;
unsigned long destroySem, querySem;
struct IPX_ECBStruct **watchdogHead, **initializeHead;
struct IPX_ECBStruct **shutdownHead, **queryHead;
CINFO *firstUnInitClient, *firstInitClient;
int shutdown = 0, shutdownOK = 0;
LONG SAPHandle;
char serverName[MAX_DN_CHARS];
LONG oldScreenID, myScreenID, debugScreenID;

extern int DestroyClient(CINFO *oldClient);

BROKER_CALL_BACK brokerCallBack;

struct NWDSImports NWDS;

int usingNDS = FALSE;


void UnloadCleanUp(int param)

{
   int ccode;
   
   param = param;

   ccode = SetDSStatusOfBroker(0);
   shutdown = 1;
   ccode = SignalLocalSemaphore(initializeSem);
   ccode = SignalLocalSemaphore(destroySem);
   ccode = SignalLocalSemaphore(querySem);
   ccode = SignalLocalSemaphore(clientListSem);

   return;
}


void ShutdownCleanUp(void)

{
   
   int ccode;
   CINFO *thisClient, *temp;

	ccode = CloseLocalSemaphore(initializeSem);
   ccode = CloseLocalSemaphore(destroySem);
   ccode = CloseLocalSemaphore(querySem);
   ccode = CloseLocalSemaphore(clientListSem);

   CancelSleepAESProcessEvent((struct AESProcessStructure *)&brokerCallBack);

   ccode = IpxCloseSocket(INITIALIZE_SOCKET);
   ccode = IpxCloseSocket(QUERY_SOCKET);

   ccode = ShutdownAdvertising(SAPHandle);


   thisClient = *initClientInfoHead;
   while(thisClient!= NULL)
   {
      _disable();
      temp = thisClient;
      thisClient = thisClient->next;
      _enable();

      DestroyClient(temp);
   }

   thisClient = *unInitClientInfoHead;
   while (thisClient!= NULL)
   {
      _disable();
      temp = thisClient;
      thisClient= thisClient->next;
      _enable();
      free(temp);
   }

#ifdef DEBUG
   ccode = DestroyScreen(debugScreenID);
#endif

   ccode = DestroyScreen(myScreenID);

   return;
}


void InitializeNDSIfPresent(void)
{
	char *NWDSImportNames[] = {NWDS_IMPORT_NAME_LIST};
	void (**importFunction)();
	int numImportNames;
	LONG NLMHandle;
	int ccode;

	numImportNames = sizeof(NWDSImportNames)/sizeof(NWDSImportNames[0]);

	// make sure our assumption about the size of our function array is correct

	assert(sizeof(NWDS) == sizeof(importFunction) * numImportNames);

	// we'll initialize our list of function pointers

	NLMHandle = GetNLMHandle();

	importFunction = (void (**)())&NWDS;
	while (numImportNames--)
	{
		if ((importFunction[numImportNames] = ImportSymbol(NLMHandle,
				NWDSImportNames[numImportNames])) == 0)
			break;
	}
	if (numImportNames)
		return;
	// if weve gotten here, then we were able to import all the symbols we need
	// to use DS

	context = NWDS.CreateContext();
	if (context == ERR_CONTEXT_CREATION)
	{
		printf("Error: couldn't create directory context\n");
		return;
	}

	printf("Enter Admin's distinguished name (eg: .CN=Admin.O=Novell) : ");
	gets(admin);
	printf("Enter Admin's password: ");
	gets(adminPassword);

	ccode = NWDS.Login(context, 0, admin, adminPassword, 0);
	if (ccode)
	{
		printf("Error: couldn't login as %s\n", admin);
		goto finalCleanUp;
	}

   ccode = CreateBrokerClass();
   if (ccode == -1)
	{
		goto finalCleanUp;
	}

   ccode = CreateBrokerObject();
   if (ccode == -1)
	{
		goto finalCleanUp;
	}

   ccode = SetDSStatusOfBroker(1);
   if (ccode == -1)
	{
		goto finalCleanUp;
	}
	usingNDS = TRUE;
	return;

finalCleanUp:
	NWDS.FreeContext(context);
	return;
}



int CreateBrokerClass(void)

{
   NWCCODE ccode;
   int err = 0, index;
   Class_Info_T classInfo;
   Buf_T * classBuffer;

   char *containmentClasses[] = {"Organization", "Organizational Unit"};

   err = 1;          /* We need to free the context upon exit */

   ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &classBuffer);
   if (ccode)
      {
      printf("\nError allocating NWDS buffer\n\t(CreateBrokerClass): %i", ccode);
      goto CleanUp;
      }
   err = 2;          /* We need to free the buffer upon exit */

   ccode = NWDS.InitBuf(context, DSV_DEFINE_CLASS, classBuffer);
   if (ccode)
      {
      printf("\nError initializing NWDS buffer\n\t(CreateBrokerClass): %i", ccode);
      goto CleanUp;
      }

	classInfo.classFlags = DS_EFFECTIVE_CLASS;
   classInfo.asn1ID.length = 0x08;
   memset(classInfo.asn1ID.data, 0x00, 8);


	// 1st class item is super class names
   ccode = NWDS.BeginClassItem(context, classBuffer);
   if (ccode)
      {
      printf("\nError beginning class item (super class): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutClassItem(context, classBuffer, "Resource");
   if (ccode )
      {
      printf("\nError putting class item (Resource): %i", ccode);
      goto CleanUp;
      }

	// 2nd class item is containment classes
   ccode = NWDS.BeginClassItem(context, classBuffer);

   if (ccode )
      {
      printf("\nError beginning class item (containment classes): %i", ccode);
      goto CleanUp;
      }

   for (index = 0; index < 2; index++)
      {
      ccode = NWDS.PutClassItem(context, classBuffer, containmentClasses[index]);
      if (ccode )
         {
         printf("\nError putting class item (%s): %i",
               containmentClasses[index], ccode);
         goto CleanUp;
         }
      }

	// 3rd class item is naming attributes
   ccode = NWDS.BeginClassItem(context, classBuffer);
   if (ccode )
      {
      printf("\nError beginning class item (naming attributes): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutClassItem(context, classBuffer, "CN");

   if (ccode)
		{
      printf("\nError putting class item (Common Name): %i", ccode);
      goto CleanUp;
      }

	// 4th class item is mandatory attributes
   ccode = NWDS.BeginClassItem(context, classBuffer);
   if (ccode )
      {
      printf("\nError beginning class item (mandatory attributes): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutClassItem(context, classBuffer, "Status");

   if (ccode)
		{
      printf("\nError putting class item (Common Name): %i", ccode);
      goto CleanUp;
      }

	// 5th class item is optional attributes
   ccode = NWDS.BeginClassItem(context, classBuffer);
   if (ccode )
      {
      printf("\nError beginning class item (optional attributes): %i", ccode);
      goto CleanUp;

      }

   ccode = NWDS.PutClassItem(context, classBuffer, "Network Address");
   if (ccode )
      {
      printf("\nError putting class item (Network Address): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.DefineClass(context, "Broker Server", &classInfo, classBuffer);
   if (ccode )
      {
      if (ccode == ERR_CLASS_ALREADY_EXISTS)
         {
         printf("\nClass already exists -- not defined");
         ccode = 0;
         }
      else
         printf("\nError defining NWDS class (Broker Server): %i", ccode);
      }
		return 0;

CleanUp:

   switch(err)
      {
      case 2:
         NWDS.FreeBuf(classBuffer);
      default:
         break;
      }
   if (ccode )
      return(-1);
   return(0);
}



int CreateBrokerObject(void)

{

   NWCCODE ccode;
   int err = 0, index;
   Buf_T * objectBuffer;
   char *attrNames[] = {"Object Class", "Host Server", "Status",
                        "Network Address", "Description"};
   uint32 attrSyntaxes[] = {SYN_CLASS_NAME, SYN_DIST_NAME,
                            SYN_INTEGER, SYN_NET_ADDRESS,
                            SYN_PR_STRING};

   Class_Name_T objClass = "Broker Server";
   char name[48 + 3];
   Integer_T stat = 0;
   Net_Address_T address;
   PR_String_T desc = "NetWare Disk Broker Service";
   uint8 actualAddress[12];

	name[0] = 'C';
	name[1] = 'N';
	name[2] = '=';
	GetFileServerName(0, &name[3]);

   ccode = IpxGetInternetworkAddress(actualAddress);
   if (ccode)
      {
      printf("\nError retrieving IPX address\n\t(CreateBrokerObject): %i", ccode);
      return(-1);
      }

   address.addressType = NT_IPX;
   address.addressLength = 12;
   address.address = actualAddress;

   err = 1;          /* need to free context upon exit */

   ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &objectBuffer);
   if (ccode )
      {
      printf("\nError allocating NWDS buffer\n\t(CreateBrokerObject): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.InitBuf(context, DSV_ADD_ENTRY, objectBuffer);
   if (ccode )
      {
      printf("\nError initializing NWDS buffer\n\t(CreateBrokerObject): %i", ccode);
      goto CleanUp;
      }
   err = 2;

   for (index = 0; index < 5; index++)
      {
      ccode = NWDS.PutAttrName(context, objectBuffer, attrNames[index]);
      if (ccode )
         {
         printf("\nError putting attribute name (%s) in buffer\n\t(CreateBrokerObject): %i",
               attrNames[index], ccode);
         goto CleanUp;
         }
      switch(index)
         {
         case 0:
            {
            ccode = NWDS.PutAttrVal(context,
                                   objectBuffer,
                                   attrSyntaxes[index],
                                   objClass);
            break;
            }
         case 1:
            {
            ccode = NWDS.PutAttrVal(context,
                                   objectBuffer,
                                   attrSyntaxes[index],
                                   name);
            break;
            }
         case 2:
            {
            ccode = NWDS.PutAttrVal(context,
                                   objectBuffer,
                                   attrSyntaxes[index],
                                   &stat);
            break;
            }
          case 3:
            {
            ccode = NWDS.PutAttrVal(context,
                                   objectBuffer,
                                   attrSyntaxes[index],
                                   &address);
            break;
            }
          case 4:
            {
            ccode = NWDS.PutAttrVal(context,
                                   objectBuffer,
                                   attrSyntaxes[index],
                                   desc);
            break;
            }
          default:
            {
            printf("\nError: index out of range on NWDSPutAttrVal");
            goto CleanUp;
            }
         }
      if (ccode )
         {
         printf("\nError putting attribute value (%) in buffer\n\t(CreateBrokerObject): %i",
               attrNames[index], ccode);
         goto CleanUp;
         }
      }


   ccode = NWDS.AddObject(context, "CN=BrokerServer", NULL, 0, objectBuffer);
   if (ccode )
      {
      if (ccode == ERR_ENTRY_ALREADY_EXISTS)
		{
         printf("\nObject already exists -- not added");
		   ccode = 0;
		}

      else
         {
         printf("\nError adding object to Directory\n\t(CreateBrokerObject): %i", ccode);
         goto CleanUp;
         }
      }

CleanUp:

   switch(err)
      {
      case 2:
         NWDS.FreeBuf(objectBuffer);
      default:
         break;
      }

   if (ccode )
      return(-1);
   return(0);
}




int SetDSStatusOfBroker(int changedStatus)

{


   /* remember to initialize the host server attribute here */

   NWCCODE ccode;
   int err = 0, index;
   Buf_T *changeBuffer, *attrBuffer, *nameBuffer;
   uint32 infoType = 1, attrCount, attrValCount, syntaxID;
   int32 iterationHandle = -1;
   Integer_T stat, oldStat;
   Net_Address_T address,oldAddress;
   uint8 actualAddress[12], previousAddress[12];
   char retrievedAttrName[MAX_DN_CHARS];

   stat = changedStatus;
   ccode = IpxGetInternetworkAddress(actualAddress);
   if (ccode)
      {
      printf("\nError retrieving IPX address: %i", ccode);
      return(-1);
      }

   address.addressType = NT_IPX;
   address.addressLength = 12;
   address.address = actualAddress;
   oldAddress.address = previousAddress;
   stat = changedStatus;

   err = 1;

   ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &changeBuffer);
   if (ccode )
      {
      printf("\nError allocating NWDS buffer (changeBuffer)\n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }
   err = 2;


   ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &attrBuffer);
   if (ccode )
      {
      printf("\nError allocating NWDS buffer (attrBuffer)\n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }
   err = 3;

   ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &nameBuffer);
   if (ccode )
      {
      printf("\nError allocating NWDS buffer (nameBuffer)\n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }
   err = 4;

   ccode = NWDS.InitBuf(context, DSV_READ, nameBuffer);
   if (ccode )
      {
      printf("\nError initializing NWDS buffer (nameBuffer)\n\t \
            (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrName(context, nameBuffer, "Network Address");
   if (ccode )
      {
      printf("\nError putting name (Network Address) in NWDS buffer\n\t \
            (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrName(context, nameBuffer, "Status");
   if (ccode )
      {
      printf("\nError putting name (Status) in NWDS buffer\n\t \
            (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }


   ccode = NWDS.Read(context, "CN=BrokerServer", infoType,
                    0, nameBuffer, &iterationHandle, attrBuffer);
   if (ccode )
      {
      printf("\nError reading object info (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.GetAttrCount(context, attrBuffer, &attrCount);
   if (ccode )
      {
      printf("\nError getting attribute count (SetDSStatusOfBroker): \
               %i", ccode);
      goto CleanUp;
      }

   for(index = 0; attrCount; index++, attrCount--)
      {
      ccode = NWDS.GetAttrName(context, attrBuffer, retrievedAttrName,
                              &attrValCount, &syntaxID);
      if (ccode )
         {
         printf("\nError getting attr. name (count: %i) \
                  (SetDSStatusOfBroker): %i", index, ccode);
         goto CleanUp;
         }
      switch(syntaxID)
         {
         case SYN_INTEGER:
            {
            if (!strcmp(retrievedAttrName, "Status"))
               {
               ccode = NWDS.GetAttrVal(context,
                                        attrBuffer,
                                        syntaxID,
                                        &oldStat);
               if (ccode )
                  {
                  printf("Error retrieving attribute value \
                           (status) (count: %i) (SetDSStatusOfBroker): %i",
                           index, ccode);
                  goto CleanUp;
                  }
               }
            break;
            }

         case SYN_NET_ADDRESS:
            {
            if (!strcmp(retrievedAttrName, "Network Address"))
               {
               ccode = NWDS.GetAttrVal(context,
                                        attrBuffer,
                                        syntaxID,
                                        &oldAddress);
               if (ccode )
                  {
                  printf("Error retrieving attribute value \
                           (network address) (count: %i) \
                           (SetDSStatusOfBroker): %i", 
                           index, ccode);
                  goto CleanUp;
                  }
               }
            break;
            }
         default:
            {
            printf("\nError: invalid attribute retrieved \
                  (count: %i) (SetDSStatusOfBroker)", index);
            goto CleanUp;
            }
         }
      }

                                        

   ccode = NWDS.InitBuf(context, DSV_MODIFY_ENTRY, changeBuffer);
   if (ccode )
      {
      printf("\nError initializing NWDS buffer (changeBuffer)\n\t \
            (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutChange(context, changeBuffer, DS_REMOVE_VALUE, "Network Address");
   if (ccode )
      {
      printf("\nError putting change record (network address 1) in NWDS buffer \
               \n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrVal(context, changeBuffer, SYN_NET_ADDRESS, &oldAddress);
   if (ccode )
      {
      printf("\nError putting old value (network address 1) in NWDS buffer\n\t \ 
               (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutChange(context, changeBuffer, DS_ADD_VALUE, "Network Address");
   if (ccode )
      {
      printf("\nError putting change record (network address 2) in NWDS buffer \
               \n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrVal(context, changeBuffer, SYN_NET_ADDRESS, &address);
   if (ccode )
      {
      printf("\nError putting new value (network address 2) in NWDS buffer\n\t \ 
               (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutChange(context, changeBuffer, DS_REMOVE_VALUE, "Status");
   if (ccode )
      {
      printf("\nError putting change record (status 1) in NWDS buffer \
               \n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrVal(context, changeBuffer, SYN_INTEGER, &oldStat);
   if (ccode )
      {
      printf("\nError putting old value (status 1) in NWDS buffer\n\t \ 
               (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutChange(context, changeBuffer, DS_ADD_VALUE, "Status");
   if (ccode )
      {
      printf("\nError putting change record (status 2) in NWDS buffer \
               \n\t(SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.PutAttrVal(context, changeBuffer, SYN_INTEGER, &stat);
   if (ccode )
      {
      printf("\nError putting new value (status 2) in NWDS buffer\n\t \ 
               (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

   ccode = NWDS.ModifyObject(context, "CN=BrokerServer", NULL, 0, changeBuffer);
   if (ccode )
      {
      printf("\nError modifying object (SetDSStatusOfBroker): %i", ccode);
      goto CleanUp;
      }

CleanUp:

   switch(err)
      {
      case 4:
         NWDS.FreeBuf(nameBuffer);
      case 3:
         NWDS.FreeBuf(attrBuffer);
      case 2:
         NWDS.FreeBuf(changeBuffer);
      default:
         break;
      }
   if (ccode )
      return(-1);
   return(0);
}


int main(int argc, char **argv)

{
   int ccode;

   signal(SIGTERM, UnloadCleanUp);
   
	ccode = AtUnload(ShutdownCleanUp);

   ccode = RenameThread(GetThreadID(), "Broker Main");

   if (argc < 2)
	{
      GetFileServerName(0, serverName);
	}

   else
	{
      strncpy(serverName, argv[1], 48);
	}

	(void)mkdir("sys:clients");

	InitializeNDSIfPresent();
	if (usingNDS)
	{
		printf("Broker Server is using NetWare Directory Services\n");
	}
	else
	{
		printf("Broker Server is NOT using NetWare Directory Services since it\n"
				"is either not available or can not be initialized\n");
	}

   SAPHandle = AdvertiseService(SVC_TYPE, serverName, QUERY_SOCKET);
   
   ccode = BeginThreadGroup(ConnectMain, NULL, 0x4000, NULL);

   while(!shutdown)
	{

		/* This is a good place to put a console display up on the server */
      /* showing active clients, disk usage, and so on. */

      delay(1);
      ThreadSwitch();
	}

   return(0);
}

