/*************************************************************************
*	DIR.NLM																					 *
*																								 *
*  NLM version of the DOS dir command                                	 *
*------------------------------------------------------------------------*
*																								 *
*	filename		:	dir.c  															    *
*	date    		:  Oct 30 1992														 	 *
*	version 		:	1.0																	 *
*	compiler		:  Watcom C 386 (see makefile for cc options)				 *
*																								 *
*	Copyright 1992, 1993 Michael Day. This work (and portions of it)				 *
*	may be freely copied, altered, and distributed, either in source		 *
*	or binary form, provided that it is not used for commercial purposes, *
*	except as a teaching tool to demonstrate network programming.			 *
*																								 *
*	Michael Day																				 *
*	(801) 429-3213																			 *
*	CIS 41670,475																			 *
*                                                                        *
*  CHANGES:                                                              *
*                                                                        *
*     03/22/92 Some paths in DisplayData would return without            *
*              decrementing the global recursionLevel variable.          *
*              Program would abort even though there was plenty of       *
*              stack remaining. (Only showed up on volumes with lots     *
*              of subdirectories.)                                       *
*                                                                        *
*              Fixed by adding a decrement at every exit point.          *
*																								 *
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nwdir.h>
#include <nwmisc.h>
#include <nwfile.h>
#include <errno.h>
#include <dos.h>
#include <conio.h>
#include <time.h>
#include <nwbindry.h>
#include <process.h>

#define SUBDIRECTORY_BIT				0x00000010
#define MAX_RECURSION               50

/***** GLOBAL FORMAT FLAGS *****/

BYTE displayAllFlag = 0;
BYTE displayWideFlag = 0;
BYTE displayPauseFlag = 0;
BYTE displayOwnerFlag = 0;
BYTE displayUpdaterFlag = 0;
BYTE logFileFlag = 0;
BYTE displaySubDirFlag = 0;
BYTE helpFlag = 0;


/***** OTHER GLOBALS *****/
FILE *logFile;
BYTE logFilePath[_MAX_PATH + 1];
BYTE recursionLevel = 0;

void usage(void)

{
   printf("\nUSAGE: DIR [path] [-options] [-l<log file name>]\n\n");
   printf("OPTIONS: A W P O U H S\n");
   printf("\t-A to display all file information\n");
   printf("\t-W to display in a wide format\n");
   printf("\t-P to pause display on each screen\n");
   printf("\t-O to display file owner\n");
   printf("\t-U to display last updater\n");
   printf("\t-H to display this help message\n");
   printf("\t-S to display file information for subdirectories\n\n");
   printf("When you activate the -L option, all output is also\n");
   printf("dumped to the log file you specify\n");
   printf("\n");
   return;
}




void DisplayData(char *startingPath)

{
   char buf[81], timeBuf[21], timeBuf2[21];
   char owner[49], updater[49];
   char path[_MAX_PATH + 1];
   char tempPath[_MAX_PATH + 1];
   char server[_MAX_SERVER + 1];
   char volume[_MAX_VOLUME + 1];
   char directories[_MAX_DIR + 1];
   char *p;
   DIR *fileBuf, *tempFileBuf, *dirBuf, *tempDirBuf;
   WORD column = 0, row = 0, objectType;
   int ccode;
   time_t timePtr;
   struct tm lTimePtr;
   LONG updateTime;


   recursionLevel++;
   clrscr();
   ccode = ParsePath(startingPath, server, volume, directories);
   if (ccode)
      {
      recursionLevel--;
      return;
      }

   sprintf(path, "%s/%s:%s", server, volume, directories);

   fileBuf = tempFileBuf = opendir(startingPath);
   if (!fileBuf)
      {
      if (displaySubDirFlag)
         goto CheckSubDirs;
      recursionLevel--;
      return;
      }

   else
      {
      gotoxy(column, row);
      printf("%s", path);
      if (logFileFlag)
         fprintf(logFile, "\n\n%s\n", path);
      row += 2;
      }
   while (tempFileBuf)
      {
      tempFileBuf = readdir(tempFileBuf);
      if (!tempFileBuf)
         break;

      if (displayWideFlag)
         {
         if (row > 22)
            {
            if (displayPauseFlag)
               {
               gotoxy(0, 24);
               PressAnyKeyToContinue();
               }
            clrscr();
            row = 2;
            }
         if (column == 75)
            {
            column = 0;
            row++;
            }
         if (tempFileBuf->d_attr & SUBDIRECTORY_BIT)
            {
            gotoxy(column, row);
            if (logFileFlag)
               fprintf(logFile, "%s", "(dir) ");
            printf("%s", "");
            }
         gotoxy(column + 1, row);
         printf("%s", tempFileBuf->d_name);
         if (logFileFlag)
            fprintf(logFile,"%s\n", tempFileBuf->d_name);
         column += 15;
         }

      else
         {
         if (row == 2)
            {
            sprintf(buf, "%-14s%-20s%-14s",
                     "File Name", "Update Date/Time", "Last Acc.", "   ");
            if (displayOwnerFlag || displayAllFlag)
               strcat(buf, "Owner         ");
            if (displayUpdaterFlag || displayAllFlag)
               strcat(buf, "Last Updater  ");
            gotoxy(column, row);
            printf("%s", buf);
            if (logFileFlag)
               fprintf(logFile, "%s\n\n", buf);
            row += 2;
            }

         column = 0;
         updateTime = (LONG)(tempFileBuf->d_date << 16);
         updateTime |= (LONG)tempFileBuf->d_time;
         timePtr = _ConvertDOSTimeToCalendar(updateTime);
         strftime(timeBuf, 20, "%m-%d-%y  %T", localtime(&timePtr));
         timePtr = _ConvertDOSTimeToCalendar(tempFileBuf->d_adatetime);
         strftime(timeBuf2, 13, "%m-%d-%y", localtime(&timePtr));

         if (displayAllFlag || displayOwnerFlag)
            {
            ccode = GetBinderyObjectName(tempFileBuf->d_uid,
                                       owner, &objectType);
            owner[15] = 0;
            }
         if (displayAllFlag || displayUpdaterFlag)
            {
            ccode = GetBinderyObjectName(tempFileBuf->d_updatedID,
                                       updater, &objectType);
            updater[15] = 0;
            }

         if (displayAllFlag)
            sprintf(buf, "%-14s%-20s%-11s%3s%-15s%-15s",
               tempFileBuf->d_name,
               timeBuf,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "--": timeBuf2,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "d  ": "   ",
               owner,
               updater);

         if (!displayAllFlag && displayOwnerFlag)
            sprintf(buf, "%-14s%-20s%-11s%3s%-15s%-15s",
               tempFileBuf->d_name,
               timeBuf,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "--": timeBuf2,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "d  ": "   ",
               owner,
               " ");

         if (!displayAllFlag && displayUpdaterFlag)
            sprintf(buf, "%-14s%-20s%-11s%3s%-15s%-15s",
               tempFileBuf->d_name,
               timeBuf,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "--": timeBuf2,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "d  ": "   ",
               " ",
               updater);

         if (!displayAllFlag && !displayUpdaterFlag && !displayOwnerFlag)
            sprintf(buf, "%-14s%-20s%-11s%3s%-15s%-15s",
               tempFileBuf->d_name,
               timeBuf,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "--": timeBuf2,
               (tempFileBuf->d_attr & SUBDIRECTORY_BIT) ? "d  ": "   ",
               " ",
               " ");
         gotoxy(column, row++);
         printf("%s", buf);
         if (logFileFlag)
            fprintf(logFile, "%s\n", buf);
         if (row > 22)
            {
            if (displayPauseFlag)
               {
               gotoxy(0, 24);
               PressAnyKeyToContinue();
               }
            clrscr();
            row = 0;
            gotoxy(column, row);
            printf("%s", path);
            row += 2;
            }
         }
      }
   closedir(fileBuf);

   if (displayPauseFlag)
      {
      gotoxy(0, 24);
      PressAnyKeyToContinue();
      }

CheckSubDirs:
   if (displaySubDirFlag)
      {
      strcpy(tempPath, path);
      p = strrchr(tempPath, '/');
      *(p + 1) = 0;
      strcat(tempPath, "*.*");
      dirBuf = tempDirBuf = opendir(tempPath);
      if (!dirBuf)
         {
         recursionLevel--;
         return;
         }
      ccode = SetReaddirAttribute(tempDirBuf, _A_SUBDIR);
         while (tempDirBuf)
            {
            strcpy(tempPath, path);
            tempDirBuf = readdir(tempDirBuf);
            if (!tempDirBuf)
               break;
            p = strrchr(tempPath, '/');
            strcpy(buf, p + 1);
            *(p + 1) = 0;
            strcat(tempPath, tempDirBuf->d_name);
            strcat(tempPath, "/");
            strcat(tempPath, buf);
            if (recursionLevel == MAX_RECURSION)
               {
               clrscr();
               printf("\nDIR.NLM: Too many recursive calls to DisplayData");
               printf("\nTry loading DIR again with a more specific path argument\n");
               PressAnyKeyToContinue();
               abort();
               }
            DisplayData(tempPath);
            }
         closedir(dirBuf);
      }
   clrscr();
   recursionLevel--;
   return;
}




int main(int argc, char **argv)

{
   BYTE startingPath[_MAX_PATH + 1];
   LONG i;
   char *p;


   for (i = 1; i < argc; i++)
      {
      p = strchr(argv[i], '-');
      if (p == NULL)
         {
         if (i == 1)
            {
            strncpy(startingPath, argv[1], _MAX_PATH);
            continue;
            }
         else
            {
            usage();
            return(0);
            }
         }
      else
         if (i == 1)
            strcpy(startingPath, "*.*");

      switch (*(p + 1))
         {
         case 'H':
         case 'h':
            {
            usage();
            return(0);
            }

         case 'A':
         case 'a':
            {
            displayAllFlag = 1;
            break;
            }

         case 'W':
         case 'w':
            {
            displayWideFlag = 1;
            break;
            }

         case 'P':
         case 'p':
            {
            displayPauseFlag = 1;
            break;
            }

         case 'O':
         case 'o':
            {
            displayOwnerFlag = 1;
            break;
            }

         case 'U':
         case 'u':
            {
            displayUpdaterFlag = 1;
            break;
            }

         case 'S':
         case 's':
            {
            displaySubDirFlag = 1;
            break;
            }

         case 'L':
         case 'l':
            {
            strncpy(logFilePath, p + 2, _MAX_PATH);
            logFile = fopen(logFilePath, "w+t");
            if (logFile == NULL)
               {
               printf("\nError opening log file\n");
               return(-1);
               }
            logFileFlag = 1;
            break;

            }
         default: break;
         }

      if (displayWideFlag)
         displayAllFlag = displayOwnerFlag = displayUpdaterFlag = 0;

      }

   if (!displayWideFlag && (displayOwnerFlag && displayUpdaterFlag))
      displayAllFlag = 1;

   DisplayData(startingPath);
   if (logFileFlag)
      fclose(logFile);
   return(0);

}
