/*************************************************************************
*	DSPACE.NLM																			    *
*																								 *
*  NLM version of the NetWare DSPACE utility                             *
*------------------------------------------------------------------------*
*																								 *
*	filename		:	dspace.c																 *
*	date    		:  Aug 25 1992														 	 *
*	version 		:	1.0																	 *
*	compiler		:  Watcom C 386 (see makefile for cc options)				 *
*																								 *
*	Copyright 1992, 93 Michael Day. This work (and portions of it)			 *
*	may be freely copied, altered, and distributed, either in source		 *
*	or binary form, provided that it is not used for commercial purposes, *
*	except as a teaching tool to demonstrate network programming.			 *
*																								 *
*	Michael Day																				 *
*	(801) 429-3213																			 *
*	CIS 41670,475																			 *
*																								 *
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <nwbindry.h>
#include <nwdir.h>
#include <nwenvrn.h>


#define REPORT    0
#define ADD       1
#define SET       2


typedef struct action {
   BYTE userSpec[49];
   BYTE code;
   LONG blocks;
}ACTION;

ACTION actions[10];
BYTE volNumber;


void usage(void)
{
   printf("\nUsage: DSPACE -v<volume number> -u<username> [-[a|s]<blocks>]");
   printf("\n\tOptions:");
   printf("\n\t  -v<volume number> specifies target volume number");
   printf("\n\t  -u<username> specifies target user (wildcards allowed)");
   printf("\n\t  -a<number of blocks> increases user's allowed blocks");
   printf("\n\t    by <number of blocks> over what the user is already using");
   printf("\n\t  -s<number of blocks> fixes user's allowed blocks");
   printf("\n\t    to <number of blocks> regardless of how many are already in use");
   printf("\n\n\tExamples:");
   printf("\n\t  LOAD DSPACE -v0 -u* <Enter>");
   printf("\n\t    reports SYS volume usage for all users\n");
   printf("\n\t  LOAD DSPACE -v1 -uMDAY -s512 <Enter>");
   printf("\n\t    fixes blocks on vol 1 available to user MDAY to 512\n");
   printf("\n\t  LOAD DSPACE -v0 -u* -a512 -uMDAY -s512 <Enter>");
   printf("\n\t    increases blocks on SYS available to all users");
   printf("\n\t    but then fixes MDAY's blocks to 512\n");
   printf("\n\tAll data is recorded to file SYS:DSPACE$.DAT");

   return;
}


int main(int argc, char **argv)

{
   BYTE commandIndex, index, objectName[49], volName[18];
   BYTE restrictionEnforced;
   char objectHasProperties, objectFlag, objectSecurity;
   char *argP;
   FILE *log;
   time_t currentTime;
   WORD objectType, totalBlocks, sectorsPerBlock, availableBlocks;
   WORD totalDirectorySlots, availableDirectorySlots,isRemovable;
   LONG objectID;
   LONG usedDirectories, usedFiles, usedBlocks, spaceRestriction;
   LONG systemElapsedTime, unusedBlocks, newRestriction;
   int ccode;



   if (argc == 1)
      {
      usage();
      return(0);
      }

   for(commandIndex = 0, index = 1; index < argc; index++)
      {
      if ((argP = strchr(argv[index], '-')) != NULL)
         {

         if (*(argP + 1) == 'v' || *(argP + 1) == 'V')
            {
            volNumber = (*(argP + 2) - '0');
            continue;
            }

         if (*(argP + 1) == 'u' || *(argP + 1) == 'U')
            {
            strncpy(actions[commandIndex].userSpec, strupr(argP + 2), 48);

            if (index + 1 == argc)
               continue;

            if ((argP = strchr(argv[index + 1], '-')) != NULL)
               if (*(argP + 1) == 'u' || *(argP + 1) == 'U')
                  commandIndex++;

            continue;
            }

         if (*(argP + 1) == 's' || *(argP + 1) == 'S')
            {
            actions[commandIndex].code = SET;
            actions[commandIndex].blocks = atol(argP + 2);
            commandIndex++;
            continue;
            }

         if (*(argP + 1) == 'a' || *(argP + 1) == 'A')
            {
            actions[commandIndex].code = ADD;
            actions[commandIndex].blocks = atol(argP + 2);
            commandIndex++;
            continue;
            }
         }
      else
         {
         usage();
         return(0);
         }
      }


   log = fopen("DSPACE$.DAT", "at+");
   if (!log)
      {
      printf("\nUnable to open DSPACE$.DAT: %s", strerror(errno));
      return(-1);
      }

   currentTime = time(NULL);
   fprintf(log, "\n\n\nDSPACE.NLM User Volume Space Report\t%s",
           ctime(&currentTime));


   ccode = GetVolumeInfoWithNumber(
                  volNumber,
                  volName,
                  &totalBlocks,
                  &sectorsPerBlock,
                  &availableBlocks,
                  &totalDirectorySlots,
                  &availableDirectorySlots,
                  &isRemovable);
   if (ccode)
      {
      printf("\nUnable to get volume information: %s", strerror(errno));
      fprintf(log, "\nUnable to get volume information: %s", strerror(errno));

      return(-1);
      }

   printf("\nVolume Information:\n");
   fprintf(log, "\nVolume Information:\n");
   printf("\n\tVol number: %i Vol name: %s", volNumber, volName);
   fprintf(log, "\n\tVol number: %i Vol name: %s", volNumber, volName);
   printf("\n\ttotal blocks: %i; sectors per block: %i",
            totalBlocks, sectorsPerBlock);
   fprintf(log, "\n\ttotal blocks: %i; sectors per block: %i",
            totalBlocks, sectorsPerBlock);
   printf("\n\tavailable blocks: %i; total directory slots: %i",
            availableBlocks, totalDirectorySlots);
   fprintf(log, "\n\tavailable blocks: %i; total directory slots: %i",
            availableBlocks, totalDirectorySlots);
   printf("\n\tavailable directory slots: %i", availableDirectorySlots);
   fprintf(log, "\n\tavailable directory slots: %i", availableDirectorySlots);


   for(commandIndex = 0; commandIndex < 10; commandIndex++)
      {

      if (strlen(actions[commandIndex].userSpec) == 0)
         break;

      objectID = -1;

      ccode = ScanBinderyObject(
                  actions[commandIndex].userSpec,
                  OT_USER,
                  &objectID,
                  objectName,
                  &objectType,
                  &objectHasProperties,
                  &objectFlag,
                  &objectSecurity);


      if (ccode)
         {
         printf("\nError ScanBinderyObject: %s", strerror(errno));
         fprintf(log, "\nError ScanBinderyObject: %s", strerror(errno));
         printf("\nUnable to find user on initial bindery search");
         fprintf(log, "\nUnable to find user on initial bindery search");
         printf("\nSearch spec: %s", actions[commandIndex].userSpec);
         fprintf(log, "\nSearch spec: %s", actions[commandIndex].userSpec);
         }


      while(!ccode)
         {
         ccode = GetDiskUtilization(
                     objectID,
                     volNumber,
                     &usedDirectories,
                     &usedFiles,
                     &usedBlocks);
         if (ccode)
            {
            printf("\nUnable to get user disk utilization: %s",
                     strerror(errno));
            fprintf(log, "\nUnable to get user disk utilization: %s",
                     strerror(errno));

            return(-1);
            }

         ccode = GetBinderyObjectDiskSpaceLeft(
                     0,
                     objectID,
                     &systemElapsedTime,
                     &unusedBlocks,
                     &restrictionEnforced);

         if (ccode)
            {
            printf("\nUnable to get user space restriction: %s",
                     strerror(errno));
            fprintf(log, "\nUnable to get user space restriction: %s",
                     strerror(errno));

            return(-1);
            }

         printf("\n\nINFORMATION FOR USER %s", objectName);
         fprintf(log, "\n\nINFORMATION FOR USER %s", objectName);
         printf("\n\tDirectories in use: %li; files in use: %li",
               usedDirectories, usedFiles);
         fprintf(log, "\n\tDirectories in use: %li; files in use: %li",
               usedDirectories, usedFiles);
         printf("\n\tblocks in use: %li; KB in use: %li",
               usedBlocks, ((usedBlocks * sectorsPerBlock) / 2));
         fprintf(log, "\n\tblocks in use: %li; KB in use: %li",
               usedBlocks, ((usedBlocks * sectorsPerBlock) / 2));

         switch(actions[commandIndex].code)
            {
            case REPORT:
               break;

            case SET:
               {

               newRestriction = actions[commandIndex].blocks;

               ccode = AddUserSpaceRestriction(
                           volNumber,
                           objectID,
                           newRestriction);
               if (ccode)
                  {
                  printf("\nUnable to add user space restriction: %s",
                         strerror(errno));
                  fprintf(log, "\nUnable to add user space restriction: %s",
                         strerror(errno));
                  }

               else
                  {
                  printf("\nNEW USER RESTRICTION: %li blocks %s",
                            newRestriction,
                            restrictionEnforced ? "(enforced)" :
                            "(not enforced)");
                  fprintf(log, "\n\nNEW USER RESTRICTION: %li blocks %s",
                            newRestriction,
                            restrictionEnforced ? "(enforced)" :
                            "(not enforced)");
                  }
               break;
               }

            case ADD:
               {
               newRestriction = usedBlocks;
               newRestriction += actions[commandIndex].blocks;

               ccode = AddUserSpaceRestriction(
                           volNumber,
                           objectID,
                           newRestriction);

               if (ccode)
                  {
                  printf("\nUnable to add user space restriction: %s",
                         strerror(errno));
                  fprintf(log, "\nUnable to add user space restriction: %s",
                         strerror(errno));
                  }

               else
                  {
                  printf("\n\nNEW USER RESTRICTION: %li blocks %s",
                            newRestriction,
                            restrictionEnforced ? "(enforced)" :
                            "(not enforced)");
                  fprintf(log, "\n\nNEW USER RESTRICTION: %li blocks %s",
                            newRestriction,
                            restrictionEnforced ? "(enforced)" :
                            "(not enforced)");
                  }
               break;
               }
            default: break;
            }


         ccode = ScanBinderyObject(
                     actions[commandIndex].userSpec,
                     OT_USER,
                     &objectID,
                     objectName,
                     &objectType,
                     &objectHasProperties,
                     &objectFlag,
                     &objectSecurity);

         }  // while (!ccode)
      }  //   for(commandIndex = 0; commandIndex < 10; commandIndex++)

   fclose(log);
   return(0);
}

