/*************************************************************************
*	FILEIO.NLM																			    *
*																								 *
*  Special test NLM to guage performance of advanced file I/O APIs       *
*------------------------------------------------------------------------*
*																								 *
*	filename		:	fileio.c																 *
*	date    		:  Nov 14 1992														 	 *
*	version 		:	2.0																	 *
*	compiler		:  Watcom C 386 (see makefile for cc options)				 *
*																								 *
*	Copyright 1992, 93 Michael Day. This work (and portions of it)	   	 *
*	may be freely copied, altered, and distributed, either in source		 *
*	or binary form, provided that it is not used for commercial purposes, *
*	except as a teaching tool to demonstrate network programming.			 *
*																								 *
*	Michael Day																				 *
*	CIS 41670,475																			 *
*																								 *
**************************************************************************/


#include <stdio.h>
#include <time.h>
#include <nwsemaph.h>
#include <string.h>
#include <errno.h>
#include <stddef.h>
#include <io.h>
#include <fcntl.h>
#include <share.h>
#include <sys\stat.h>
#include <nwsync.h>
#include <advanced.h>

typedef struct ioBuf {
   struct ioBuf *next;
   BYTE *buf;
}IOBUF;

IOBUF iobufs[300];

T_mwriteBufferStructure gBufs[300];
T_cacheBufferStructure *aBufs;

LONG blocksize, operations, semHandle, handle, switchFlag;

void usage(void)
{
   printf("\nUSAGE: FILEIO <blocksize> <operations> [-s]");
   printf("\n\tWhere <blocksize> is the size of each I/O operation");
   printf("\n\tin bytes, and <operations> is the number of reads");
   printf("\n\tand writes performed for each type file I/O API\n");
   printf("\n\n-s causes a ThreadSwitch after every I/O operation");
}


void cleanup(void)
{
   CloseLocalSemaphore(semHandle);
   return;
}


int main(int argc, char **argv)

{

   int index, ccode, i, bufIndex;
   LONG position, gwrites, areads;
   time_t start, end, total;
   BYTE buf[25];
   char *p;

   if (argc < 3)
      {
      usage();
      return(0);
      }

   atexit(cleanup);

   blocksize = atol(argv[1]);
   operations = atol(argv[2]);
   if (operations > 300)
      operations = 300;

   p = strchr(argv[3], '-');
   if (p != NULL)
      if (*(p + 1) == 's' || *(p + 1) == 'S')
         switchFlag = 1;
      else
         switchFlag = 0;

   semHandle = OpenLocalSemaphore(0L);

   for (index = 0; index < operations; index++)
      {
      iobufs[index].buf = (BYTE *)malloc(blocksize);
      if (!iobufs[index].buf)
         {
         printf("\nUnable to allocate working buffer");
         goto Exit;
         }
      memset(iobufs[index].buf, '1', blocksize);
      }

   for(i = 0; i < 100; i++)
      {
      sprintf(buf, "SYS:file%i.dat", i);
      handle = creat(buf, 0);
      if (handle != -1)
         break;
      }
   if (i == 99)
      {
      printf("\nUnable to open test file: %s", strerror(errno));
      goto Exit;
      }

/*************************************************************************
* WRITE TESTS                                                            *
*************************************************************************/


   /* To get a baseline, we do some normal writes */
   /* first we get a time with no record locking */

   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0; index < operations; index++)
         {
         ccode = write(handle, iobufs[index].buf, blocksize);
         if (switchFlag)
            ThreadSwitch();
         }
      ccode = lseek(handle, 0L, SEEK_SET);
      }

   end = clock();
   total = end - start;
   printf("\nwrite() no locks; %li bytes: %li hundredths of a second",
            ((blocksize * operations) * 100),
            total);

   /* Now we get a time with record locking */
   /* we set read/write exclusive locks */

   start = clock();
   for (i = 0; i < 100; i++)
      {
      position = 0;
      ccode = lseek(handle, 0L, SEEK_SET);
      for (index = 0; index < operations; index++)
         {
         ccode = LogPhysicalRecord(
                     handle,
                     position,
                     blocksize,
                     0x01,
                     18);

         ccode = write(handle, iobufs[index].buf, blocksize);

         ccode = ClearPhysicalRecord(
                     handle,
                     position,
                     blocksize);
         position += blocksize;
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;
   printf("\nwrite() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);


   /* Now that we have a baseline, try qwrite() */
   /* Note: qwrite does not maintain a file position, */
   /* so we need to do that ourselves. */

   /* get a time without record locking */
   
   start = clock();
   for (i = 0; i < 100; i++)
      {
      position = 0;
      for (index = 0; index < operations; index++)
         {
         ccode = qwrite(handle, iobufs[index].buf, blocksize, position);
         position += blocksize;
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;
   printf("\nqwrite() no locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);

   /* get a time with record locking */
   
   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0, position = 0;
           index < operations;
           index++, position += blocksize)
         {
         ccode = LogPhysicalRecord(
                     handle,
                     position,
                     blocksize,
                     0x01,
                     18);

         ccode = qwrite(handle, iobufs[index].buf, blocksize, position);

         ccode = ClearPhysicalRecord(
                     handle,
                     position,
                     blocksize);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nqwrite() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);



   close(handle);
   handle = open(buf, O_RDWR | O_BINARY);
   if (handle == -1)
      {
      printf("\nError re-opening file for async calls: %s", strerror(errno));
      goto Exit;
      }


    /* gwrite no locks */

   for (index = 0; index < operations; index++)
      {
      gBufs[index].mwriteBufferPointer = iobufs[index].buf;
      gBufs[index].mwriteBufferLength = blocksize;
      }

   start = clock();
   for (i = 0; i < 100; i++)
      {
      ccode = lseek(handle, 0L, SEEK_SET);
      ccode = gwrite(handle, &gBufs[0], operations, &gwrites);
      if (ccode != (operations * blocksize))
         printf("\nError gwrite() no locks");
      }
   end = clock();
   total = end - start;
   
   printf("\ngwrite() no locks; %li bytes: %li hundredths of a seconds",
            (blocksize * operations * 100),
            total);

   /* gwrite with locks */
   for (index = 0; index < operations; index++)
      {
      gBufs[index].mwriteBufferPointer = iobufs[index].buf;
      gBufs[index].mwriteBufferLength = blocksize;
      }
   
   start = clock();
   for (i = 0, position = 0; i < 100; i++)
      {
      ccode = lseek(handle, 0L, SEEK_SET);

      ccode = LogPhysicalRecord(
                  handle,
                  position,
                  (blocksize * operations),
                  0x01,
                  18);

      ccode = gwrite(handle, &gBufs[0], operations, &gwrites);

      if (ccode != (operations * blocksize))
         printf("\nError gwrite() with locks");

      ccode = ClearPhysicalRecord(
                  handle,
                  position,
                  (blocksize * operations));

      }
   end = clock();
   total = end - start;

   printf("\ngwrite() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);


/*************************************************************************
* READ TESTS                                                             *
*************************************************************************/



   /* to get a baseline, we do tests using the normal read() API */

   /* first with no record locking */
   
   start = clock();
   for (i = 0; i < 100; i++)
      {
      ccode = lseek(handle, 0L, SEEK_SET);
      for (index = 0; index < operations; index++)
         {
         ccode = read(handle, iobufs[index].buf, blocksize);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nread() no locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);


   /* again with record locks. This time we set read-only locks */

   start = clock();
   for (i = 0; i < 100; i++)
      {
      ccode = lseek(handle, 0L, SEEK_SET);
      for (index = 0, position = 0;
           index < operations;
           index++, position += blocksize)
         {
         ccode = LogPhysicalRecord(
                     handle,
                     position,
                     blocksize,
                     0x03,
                     18);
         ccode = read(handle, iobufs[index].buf, blocksize);

         ccode = ClearPhysicalRecord(
                     handle,
                     position,
                     blocksize);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nread() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);


   /* Now get a time for qread() no locks */

   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0, position = 0;
           index < operations;
           index++, position += blocksize)
         {
         ccode = qread(handle, iobufs[index].buf, blocksize, position);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nqread() no locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);

   /* time qread() with locks */

   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0, position = 0;
           index < operations;
           index++, position += blocksize)
         {
         ccode = LogPhysicalRecord(
                     handle,
                     position,
                     blocksize,
                     0x03,
                     18);

         ccode = qread(handle, iobufs[index].buf, blocksize, position);

         ccode = ClearPhysicalRecord(
                     handle,
                     position,
                     blocksize);

         if (switchFlag)
            ThreadSwitch();
         }
      }
   
   end = clock();
   total = end - start;

   printf("\nqread() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);

   
   /* Async read no locks */

   close(handle);
   handle = open(buf, O_RDWR | O_BINARY);
   if (handle == -1)
      {
      printf("\nError re-opening file for async calls: %s", strerror(errno));
      goto Exit;
      }

   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0, position = 0;
           index < operations;
           index ++, position += blocksize)
         {
         ccode = AsyncRead(
                  handle,
                  position,
                  blocksize,
                  &areads,
                  semHandle,
                  &aBufs,
                  &gwrites);

         ccode = WaitOnLocalSemaphore(semHandle);
         AsyncRelease(aBufs);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nAsyncRead() no locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);

   /* AsyncRead with locks */
   
   start = clock();
   for (i = 0; i < 100; i++)
      {
      for (index = 0, position = 0;
           index < operations;
           index++, position += blocksize)
         {
         ccode = LogPhysicalRecord(
                  handle,
                  position,
                  blocksize,
                  0x03,
                  18);

         ccode = AsyncRead(
                  handle,
                  position,
                  blocksize,
                  &areads,
                  semHandle,
                  &aBufs,
                  &gwrites);
   
         ccode = WaitOnLocalSemaphore(semHandle);
         ccode = ClearPhysicalRecord(
                  handle,
                  position,
                  blocksize);
         AsyncRelease(aBufs);
         if (switchFlag)
            ThreadSwitch();
         }
      }
   end = clock();
   total = end - start;

   printf("\nAsyncRead() with locks; %li bytes: %li hundredths of a second",
            (blocksize * operations * 100),
            total);


Exit:
 
   close(handle);
   unlink(buf);
   for (index = 0; index < operations; index++)
      if (iobufs[index].buf != NULL)
         free(iobufs[index].buf);

   return(0);
}
