/*************************************************************************
*	FLAG.NLM																					 *
*																								 *
*  NLM version of the NetWare FLAG utility (no subdirectory scanning)	 *
*------------------------------------------------------------------------*
*																								 *
*	filename		:	flag.c																 *
*	date    		:  Aug 22 1992														 	 *
*	version 		:	1.0																	 *
*	compiler		:  Watcom C 386 (see makefile for cc options)				 *
*																								 *
*	Copyright 1992, Michael Day. This work (and portions of it)				 *
*	may be freely copied, altered, and distributed, either in source		 *
*	or binary form, provided that it is not used for commercial purposes, *
*	except as a teaching tool to demonstrate network programming.			 *
*																								 *
*	Michael Day																				 *
*	(801) 429-3213																			 *
*	CIS 41670,475																			 *
*																								 *
**************************************************************************/


/***** Define the attribute values *****/

#define READ_ONLY_BIT					0x00000001
#define HIDDEN_BIT						0x00000002
#define SYSTEM_BIT						0x00000004
#define EXECUTE_BIT						0x00000008
#define SUBDIRECTORY_BIT				0x00000010
#define ARCHIVE_BIT						0x00000020
#define SHAREABLE_BIT					0x00000080	
#define TRANSACTION_BIT					0x00001000
#define IMMEDIATE_PURGE_BIT			0x00010000
#define RENAME_INHIBIT_BIT				0x00020000
#define DELETE_INHIBIT_BIT				0x00040000
#define COPY_INHIBIT_BIT				0x00080000



#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nwdir.h>
#include <nwmisc.h>
#include <nwfile.h>
#include <errno.h>



void usage(void)
{
	printf("\nUSAGE: FLAG [path [ option | [+|-] attribute(s)] ]\n\n");
	printf("386 Attributes:\n");
	printf("--------------\nD Subdirectory\n");
	printf("RO Read Only\nRW Read Write\nS Shareable\nH Hidden\nSy System\n");
	printf("T Transactional\nP Purge\nA Archive Needed\n");
	printf("CI Copy Inhibit\nDI Delete Inhibit\nRI Rename Inhibit\n");

	return;
}





int main(int argc, char **argv)

{
	LONG onAttr = 0L, offAttr = 0xffffffff, tempDate, tempCreate, tempBackup;
   WORD tempAccess;
	BYTE buf[81], path[_MAX_PATH], i, addFlag, removeFlag;
	BYTE server[_MAX_SERVER], volume[_MAX_VOLUME], dirs[_MAX_DIR];
   BYTE pauseFlag = 0;
	DIR *dirBuf, *tempDirBuf, *confirmDir, *tempConfirmDir;
	char *argP;
	int ccode = 0;


	/* process command-line options */

	if(argc > 1)
		{

		/* check for help screen option */

	   argP = strchr(argv[1], '?');
		if (argP)
			{
			argP--;
			if (*argP == '/' || *argP == '-')
				if (strlen(argP) == 2)
					{
					usage();
					return(1);
					}
			}

		/* not a help screen command line, so continue */

		strcpy(path, argv[1]);

		/* now collect the flag options if there are any */

		for (i = 2, addFlag = 0, removeFlag = 0;
			  i < argc;
			  i++, addFlag = 0, removeFlag = 0)
			{

			argP = argv[i];
			if (*argP == '+')
				addFlag++;
			if (*argP == '-')
				removeFlag++;
			if (strlen(argP) == 1)
				{
				i++;
				argP = argv[i];
				}
			else
				{
				argP++;
				while(isspace(*argP) && *argP++)
					;
				if (isspace(*argP) || !*argP)
					{
					i++;
					argP = argv[i];
					}
				}
			switch(*argP)
				{
				case 'r':
				case 'R':
					{
					if (*(argP + 1) == 'o' || *(argP + 1) == 'O')
						if (addFlag)
							onAttr |= READ_ONLY_BIT;
						if (removeFlag)
                     {
							offAttr &= ~READ_ONLY_BIT;
							offAttr &= ~RENAME_INHIBIT_BIT;
                     offAttr &= ~DELETE_INHIBIT_BIT;
                     }
					if (*(argP + 1) == 'w' || *(argP + 1) == 'W')
						if (addFlag)
                     {
							offAttr &= ~READ_ONLY_BIT;
							offAttr &= ~RENAME_INHIBIT_BIT;
                     offAttr &= ~DELETE_INHIBIT_BIT;
                     }
						if (removeFlag)
							onAttr |= READ_ONLY_BIT;
					if (*(argP + 1) == 'i' || *(argP + 1) == 'I')
						if (addFlag)
							onAttr |= RENAME_INHIBIT_BIT;
						if (removeFlag)
							offAttr &= ~RENAME_INHIBIT_BIT;
					break;
					}

				case 's':
				case 'S':
					{
					if (*(argP + 1) == 'y' || *(argP + 1) == 'Y')
						if (addFlag)
							onAttr |= SYSTEM_BIT;
						if (removeFlag)
							offAttr &= ~SYSTEM_BIT;
					else
						if (addFlag)
							onAttr |= SHAREABLE_BIT;
						if (removeFlag)
							offAttr &= ~SHAREABLE_BIT;
					break;
					}
				case 'h':
				case 'H':
					{
					if (addFlag)
						onAttr |= HIDDEN_BIT;
					if (removeFlag)
						offAttr &= ~HIDDEN_BIT;
					break;
					}
				case 't':
				case 'T':
					{
					if (addFlag)
						onAttr |= TRANSACTION_BIT;
					if (removeFlag)
						offAttr &= ~TRANSACTION_BIT;
					break;
					}
				case 'p':
				case 'P':
					{
					if (addFlag)
						onAttr |= IMMEDIATE_PURGE_BIT;
					if (removeFlag)
						offAttr &= ~IMMEDIATE_PURGE_BIT;
					break;
					}
				case 'a':
				case 'A':
					{
					if (addFlag)
						onAttr |= ARCHIVE_BIT;
					if (removeFlag)
						offAttr &= ~ARCHIVE_BIT;
					break;
					}
				case 'c':
				case 'C':
					{
					if (*(argP + 1) == 'i' || *(argP + 1) == 'I')
						if (addFlag)
							onAttr |= COPY_INHIBIT_BIT;
						if (removeFlag)
							offAttr &= ~COPY_INHIBIT_BIT;
					break;
					}
				case 'd':
				case 'D':
					{
					if (*(argP + 1) == 'i' || *(argP + 1) == 'I')
						if (addFlag)
							onAttr |= DELETE_INHIBIT_BIT;
						if (removeFlag)
							offAttr &= ~DELETE_INHIBIT_BIT;
					break;
					}
				default:
					break;
				}
			} /* for i < argc process flag options */
		}
	else
      sprintf(path, "*.*");

	/* Now we need to get file information for the path option */
	/* Note that if the user specified no flag options, the */
	/* variable attr will be equal to zero */

	tempDirBuf = dirBuf = opendir(path);
	if (!tempDirBuf)
		{
		printf("\nNo matching files found!");
		return(-1);
		}
	tempConfirmDir = confirmDir = opendir(path);
	if (!tempConfirmDir)
		{
		printf("\nUnable to allocate confirmation buffer!");
		return(-1);
		}

	while(tempDirBuf)
		{
		tempDirBuf = readdir(tempDirBuf);
		if (!tempDirBuf)
			break;
		if (onAttr)
			tempDirBuf->d_attr |= onAttr;
		if (offAttr != 0xffffffff)
			tempDirBuf->d_attr &= offAttr;

		ParsePath(path, server, volume, dirs);
		argP = strrchr(dirs, '/');
		if (argP)
			*argP = 0;
		sprintf(path, "%s/%s:%s/%s",
							server, volume, dirs,
							tempDirBuf->d_name);
      if (tempDirBuf->d_attr & SUBDIRECTORY_BIT)
         ;
      else
         {

         tempDate = (LONG)tempDirBuf->d_date;
         tempDate |= (LONG)(tempDirBuf->d_time << 16);
         tempAccess = (WORD)(tempDirBuf->d_adatetime >> 16);
         tempCreate = (WORD)(tempDirBuf->d_cdatetime >> 16);
         tempCreate |= (tempDirBuf->d_cdatetime << 16);
         tempBackup = (WORD)(tempDirBuf->d_bdatetime >> 16);
         tempBackup |= (tempDirBuf->d_bdatetime << 16);
			ccode = SetFileInfo(
							path,
							0x06,
							tempDirBuf->d_attr,
							(char *)&tempCreate,
							(char *)&tempAccess,
							(char *)&tempDate,
							(char *)&tempBackup,
							tempDirBuf->d_uid);
			if (ccode)
				printf("\nError setting file attribute: %s %s",
							strerror(errno), path);
         }

		tempConfirmDir = readdir(tempConfirmDir);
		printf("\n%-20s[ %s %s %s %s %s %s %s %s %s %s %s %s]",
			tempConfirmDir->d_name,
			(tempConfirmDir->d_attr & SUBDIRECTORY_BIT)   ? "D" : "-",
			(tempConfirmDir->d_attr & READ_ONLY_BIT) 		 ? "RO" : "--",
			(tempConfirmDir->d_attr & READ_ONLY_BIT) 		 ? "--" : "Rw",
			(tempConfirmDir->d_attr & SHAREABLE_BIT) 		 ? "S" : "-",
			(tempConfirmDir->d_attr & HIDDEN_BIT)    		 ? "H" : "-",
			(tempConfirmDir->d_attr & SYSTEM_BIT)    		 ? "SY" : "--",
			(tempConfirmDir->d_attr & TRANSACTION_BIT)	 ? "T" : "-",
			(tempConfirmDir->d_attr & IMMEDIATE_PURGE_BIT)? "P" : "-",
			(tempConfirmDir->d_attr & ARCHIVE_BIT)        ? "A" : "-",
			(tempConfirmDir->d_attr & COPY_INHIBIT_BIT)   ? "CI" : "--",
			(tempConfirmDir->d_attr & DELETE_INHIBIT_BIT) ? "DI" : "--",
			(tempConfirmDir->d_attr & RENAME_INHIBIT_BIT) ? "RI" : "--");

		}
	printf("\n");

	closedir(dirBuf);
	closedir(confirmDir);
	return(0);
}

