/*************************************************************************
*	FMON.NLM	    																		    *
*																								 *
*  Demonstration NLM: File Monitor Interface                             *
*------------------------------------------------------------------------*
*																								 *
*	filename		:	fmon.c																 *
*	date    		:  Oct 8 1992														 	 *
*	version 		:	2.0																	 *
*	compiler		:  Watcom C 386 (see makefile for cc options)				 *
*																								 *
*	Copyright 1992, Michael Day. This work (and portions of it)				 *
*	may be freely copied, altered, and distributed, either in source		 *
*	or binary form, provided that it is not used for commercial purposes, *
*	except as a teaching tool to demonstrate network programming.			 *
*																								 *
*	Michael Day																				 *
*	CIS 41670,475																			 *
*                                                                        *              
*   CHANGES:                                                             *
*           MAR 23 1993 CHANGED CALL FEMapHandleToVolumeAndDirectory     *
*                       TO NEW API FEMapConnsHandleToVolumeAndDirectory  *
*                                                                        *
*                       THIS IS A NEW API THAT HANDLES OS FILE HANDLES   *
*                       ALONG WITH CLIB FILE HANDLES                     *
*                                                                        *
**************************************************************************/


#include <stdio.h>
#include <time.h>
#include <nwsemaph.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <errno.h>
#include <fileengd.h>
#include "process.h"
#include <signal.h>
#include <string.h>
#include "fshooks.h"
#include "fmon.h"

/*----- external data and code -----*/


/*----- imported from HOOKS.386 -----*/

extern LONG numberNodes;
extern MonitorNode MonitorNodes[];
extern LONG (*EntryHooks[])(void *);
extern LONG bufferOverFlow;

/* globals */
            
LONG FMonSem, shutdownSem, shutdown = 0;
LONG actualLength;
LONG volume;
LONG FSHookHandles[18];
LONG FSHookReturnCodes[] = {
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
FILE *handle;
char buf[_MAX_PATH + 1];


void ConvertPath(LONG bufferSize, char *buf)

{
   char *p;
   int i = 0;

   p = buf;
   while (i < bufferSize)
      {
      if (*p < 0x21 || *p > 0x7e)
         *p = '\\';
      p++;
      i++;
      }
   *(buf + bufferSize) = 0;
   return;
}


void MonitorMain(void *data)

{
   char objectName[49], loginTime[7], volName[_MAX_PATH + 1];
   char modBuf[300];
   char *p;
   WORD objectType;
   LONG objectID, ccode, mainIndex = 0;
   LONG directoryNumber, streamNumber, DOSDirNum;
   LONG fileHandle;
   MonitorNode *monitorData;
   GenericAddress *dataP;


   data = data;

   while (!shutdown)
      {
      WaitOnLocalSemaphore(FMonSem);
      if (shutdown)
         break;

      if (mainIndex == numberNodes)
         mainIndex = 0;

      monitorData = &MonitorNodes[mainIndex];
      dataP = (GenericAddress *)&monitorData->hookData[0];

      switch (monitorData->hookNumber)
         {
         case ERASE_FILE:
            {
            printf("\n\n*** ERASE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** ERASE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);


            ccode = FEConvertDirectoryNumber(
                        dataP->member7,    // name space
                        dataP->member3,    // volume
                        dataP->member4,    // dirBase
                        0,                 // DOS name space
                        &DOSDirNum);       // DOS dirBase



            ccode = FEMapVolumeAndDirectoryToPath(
                        dataP->member3,   // volume
                        DOSDirNum,        // DOS dirBase
                        buf,              // our buffer
                        &actualLength);   // receives path length


            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nFile path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nFile path: %s:%s", volName, buf);
            break;

            }

         case OPEN_FILE:
            {
            printf("\n\n*** OPEN FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** OPEN FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);


            ccode = FEConvertDirectoryNumber(
                     dataP->member7,      // name space
                     dataP->member3,      // volume
                     dataP->member4,      // dirBase
                     0,                   // DOS name space
                     &DOSDirNum);         // DOS dirBase



            ccode = FEMapVolumeAndDirectoryToPath(
                     dataP->member3,      // volume
                     DOSDirNum,           // DOS dirBase
                     buf,
                     &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nFile path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nFile path: %s", buf);
            break;
            }

         case CREATE_FILE:
            {

            printf("\n\n*** CREATE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** CREATE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);

            ccode = FEConvertDirectoryNumber(
                     dataP->member7,      // name space
                     dataP->member3,      // volume
                     dataP->member4,      // dirBase
                     0,                   // DOS name space
                     &DOSDirNum);         // DOS dirBase
            

            ccode = FEMapVolumeAndDirectoryToPath(
                     dataP->member3,      // volume
                     DOSDirNum,           // DOS dirBase
                     buf,                 // our buffer
                     &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nFile path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nFile path: %s:%s", volName, buf);

            break;
            }


         case CREATE_AND_OPEN_FILE:
            {

            printf("\n\n*** CREATE AND OPEN FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** CREATE AND OPEN FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);
         

            ccode = FEConvertDirectoryNumber(
                     dataP->member7,      // name space
                     dataP->member3,      // volume
                     dataP->member4,      // dirBase
                     0,                   // DOS name space
                     &DOSDirNum);         // DOS dirBase


            FEMapVolumeAndDirectoryToPath(
                     dataP->member3,      // volume
                     DOSDirNum,           // DOS dirBase
                     buf,
                     &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nFile path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nFile path: %s", buf);

            break;
            }

         case RENAME_MOVE_ENTRY:

            {

            printf("\n\n*** RENAME MOVE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** RENAME MOVE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);
         
            ccode = FEConvertDirectoryNumber(
                        dataP->member7,      // name space
                        dataP->member3,      // volume
                        dataP->member4,      // dirBase
                        0,                   // DOS name space
                        &DOSDirNum);         // DOS dirBase


            ccode = FEMapVolumeAndDirectoryToPath(
                        dataP->member3,      // volume
                        DOSDirNum,           // DOS dirBase
                        buf,
                        &actualLength);
            
            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nOld File path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nOld File path: %s:%s", volName, buf);


            ccode = FEConvertDirectoryNumber(
                        dataP->member7,      // name space
                        dataP->member3,      // volume
                        dataP->member10,      // dirBase
                        0,                   // DOS name space
                        &DOSDirNum);         // DOS dirBase


            ccode = FEMapVolumeAndDirectoryToPath(
                        dataP->member3,      // volume
                        DOSDirNum,           // DOS dirBase
                        buf,
                        &actualLength);


            ConvertPath(actualLength, buf);
            printf("\nNew File path: %s:%s", volName, buf);
            fprintf(handle, "\nNew File path: %s:%s", volName, buf);
            break;
            }

         case CLOSE_FILE:
            {

            printf("\n\n*** CLOSE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** CLOSE FILE EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);


            ccode = FEMapConnsHandleToVolumeAndDirectory(
                        dataP->member1,   // connection
                        dataP->member3,   // file handle
                        &volume,
                        &directoryNumber);

            ccode = FEConvertDirectoryNumber(
                     FEGetOriginatingNameSpace(volume, directoryNumber),
                     volume,
                     directoryNumber,
                     0,
                     &DOSDirNum);
            

            ccode = FEMapVolumeAndDirectoryToPath(
                        volume,
                        DOSDirNum,
                        buf,
                        &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(volume, volName);

            printf("\nObject: %s; ID: %#08lx\n", objectName, objectID);
            printf("Closed File path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx\n", objectName, objectID);
            fprintf(handle, "Closed File path: %s:%s", volName, buf);
            break;
            }

         case CREATE_DIRECTORY:

            {

            printf("\n\n*** CREATE DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** CREATE DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
         
            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);


            ccode = FEConvertDirectoryNumber(
                        dataP->member7,      // name space
                        dataP->member3,      // volume
                        dataP->member4,      // dirBase
                        0,                   // DOS name space
                        &DOSDirNum);         // DOS dirBase

            ccode = FEMapVolumeAndDirectoryToPath(
                        dataP->member3,      // volume
                        DOSDirNum,           // DOS dirBase
                        buf,
                        &actualLength);
            
            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nNew Directory path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nNew Directory path: %s:%s", volName, buf);
            break;
            }


         case DELETE_DIRECTORY:
            {

            printf("\n\n*** DELETE DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** DELETE DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);

            ccode = FEConvertDirectoryNumber(
                     dataP->member7,      // name space
                     dataP->member3,      // volume
                     dataP->member4,      // dirBase
                     0,                   // DOS name space
                     &DOSDirNum);         // DOS dirBase
         

            ccode = FEMapVolumeAndDirectoryToPath(
                     dataP->member3,      // volume
                     DOSDirNum,           // DOS dirBase
                     buf,
                     &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nDeleted Directory path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nDeleted Directory path: %s:%s", volName, buf);

            break;
            }


         case MODIFY_DIR_ENTRY:
            {

            printf("\n\n*** MODIFY DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));
            fprintf(handle, "\n\n*** MODIFY DIRECTORY EVENT ***\t%s",
                        ctime(&monitorData->timeStamp));

            if (!dataP->member1)
               {
               strcpy(objectName, "NLM CONNECTION ZERO");
               objectID = 1;
               }
            
            else
               ccode = GetConnectionInformation(
                           dataP->member1,
                           objectName,
                           &objectType,
                           &objectID,
                           loginTime);


            ccode = FEConvertDirectoryNumber(
                        dataP->member7,   // name space
                        dataP->member3,   // volume
                        dataP->member4,   // dirBase
                        0,                // DOS name space
                        &DOSDirNum);      // DOS dirBase

            ccode = FEMapVolumeAndDirectoryToPath(
                        dataP->member3,   // volume
                        DOSDirNum,        // DOS dirBase
                        buf,
                        &actualLength);

            ConvertPath(actualLength, buf);
            GetVolumeName(dataP->member3, volName);

            printf("\nObject: %s; ID: %#08lx", objectName, objectID);
            printf("\nModified Directory path: %s:%s", volName, buf);
            fprintf(handle, "\nObject: %s; ID: %#08lx", objectName, objectID);
            fprintf(handle, "\nModified Directory path: %s:%s", volName, buf);

            strcpy(modBuf, "Modifications: ");
            if (dataP->member11 & MModifyNameBit)
               strcat(modBuf, "\n\tName");
            if (dataP->member11 & MFileAttributesBit)
               strcat(modBuf, "\n\tAttributes");
            if (dataP->member11 & MCreateDateBit)
               strcat(modBuf, "\n\tCreation date");
            if (dataP->member11 & MCreateTimeBit)
               strcat(modBuf, "\n\tCreation time");
            if (dataP->member11 & MOwnerIDBit)
               strcat(modBuf, "\n\tOwner ID");
            if (dataP->member11 & MLastArchivedDateBit)
               strcat(modBuf, "\n\tArchive date");
            if (dataP->member11 & MLastArchivedTimeBit)
               strcat(modBuf, "\n\tArchive time");
            if (dataP->member11 & MLastArchivedIDBit)
               strcat(modBuf, "\n\tArchiver ID");
            if (dataP->member11 & MLastUpdatedDateBit)
               strcat(modBuf, "\n\tUpdate date");
            if (dataP->member11 & MLastUpdatedTimeBit)
               strcat(modBuf, "\n\tUpdate time");
            if (dataP->member11 & MLastUpdatedIDBit)
               strcat(modBuf, "\n\tUpdater ID");
            if (dataP->member11 & MLastAccessedDateBit)
               strcat(modBuf, "\n\tAccess date");
            if (dataP->member11 & MInheritanceRestrictionMaskBit)
               strcat(modBuf, "\n\tInheritance Mask");
            if (dataP->member11 & MMaximumSpaceBit)
               strcat(modBuf, "\n\tMax. space");

            printf("\n%s", buf);
            fprintf(handle, "\n%s", buf);

            break;
            }

         }
      if (shutdown)
         break;
      monitorData->timeStamp = 0;
      mainIndex++;
      }
   return;
}





void UnloadCleanUp(int sig)

{
	sig = sig;

   shutdown = 1;

   if (bufferOverFlow)
      printf("\nWARNING: The FMON buffer overflowed %i times",
         bufferOverFlow);

	SignalLocalSemaphore(shutdownSem);
   SignalLocalSemaphore(FMonSem);

	return;
}




void ShutdownCleanUp(void)


{

	int ccode, i;

   for (i = 0; i < 9; i++)
      if (!FSHookReturnCodes[i])
         ccode = NWRemoveFSMonitorHook(i, FSHookHandles[i]);

	ccode = CloseLocalSemaphore(shutdownSem);
   ccode = CloseLocalSemaphore(FMonSem);

	return;
}


int main(void)

{
   LONG ccode, i;

   handle = fopen("$FMON$.DAT", "wt+");
   shutdownSem = OpenLocalSemaphore(0L);
   FMonSem = OpenLocalSemaphore(0L);

   if (handle == NULL || shutdownSem == NULL || FMonSem == NULL)
      return(-1);

   signal(SIGTERM, UnloadCleanUp);
	atexit(ShutdownCleanUp);

   for (i = 0; i < 9; i++)
      FSHookReturnCodes[i] =
         NWAddFSMonitorHook(i, EntryHooks[i], &FSHookHandles[i]);

   BeginThread(MonitorMain, NULL, 0x2000, NULL);

   WaitOnLocalSemaphore(shutdownSem);
                  
   return(0);

}
