
/**/
/*                                                                       */
/*	          IPX Based Time Query Client                               */
/*                                                                       */
/*   This routine will query the user for network and node address of    */
/*   any IPX time server.  It will then send a time request to           */
/*   that server using an IPX datagram.  If a response is received, the  */
/*   routine will set the local time to that received from the server    */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"   /* Missing nxt.h declarations */
#include "timeserv.h" /* Timeserver specific declarations */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main() 
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	time_t     *time_p;       /* used for conversion                        */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* ptr to TZ= string                          */

	IPXAddress  myaddress;    /* My network,node,socket                     */
	int         transport_time; /* Estimate of time to server               */
	int 	      valid_reply;  /* A valid time reply received                */
	IPXAddress  server;       /* Server address                             */
	WORD        serverSocket; /* Temp WORD area for server socket           */

	int         status;       /* scratch status                             */
	int         i;            /* scratch index                              */


/*   ECB and Data buffers for listening and sending                        */

	ECB         ipx_ECB;		           /* Event Control Block               */
	IPXHeader   ipx_header;            /* 1st data buf, IPX header          */
	BYTE        ipx_data[512] = {0};   /* 2cd data buf, User data           */

/*  */

/*   Get and display the old time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("\nThe time was %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (*time_zone == NULL)
		printf("Time zone was NULL\n");
	else
		printf("Time zone was %s\n\n",time_zone);

/*   Initialize the IPX function library   */

	if ((status=IPXInitialize()) != SUCCESSFUL )
		{
		printf("IPXInitialize fails, status = 0x%x, program aborted\n",status);
		return;
		}
/*
 *   Ask the user which timeserver he wants to use. We do this because
 *   advertising/geting server only works from a workstation perspective
 *   if there is an intermediate router or if the workstation implements
 *   a complete SAP handler, which is not the case in this simple example.
 */
	printf("Please enter the timeserver address\n\n");
	GetIPXAddress(&server);
	serverSocket=TIME_SERVER_SOCKET;			 /* server socket is well known */
	memcpy(server.socket,&serverSocket,2);

/*
 *   Open a dynamic socket for the client to use by specifying 0.
 *   The actual socket allocated is returned to us. We open this socket
 *   as SHORT_LIVED, it will close automatically on program termination.
 */
	memset(myaddress.socket,0,2);
	if ((status=IPXOpenSocket(&myaddress.socket,SHORT_LIVED)) != SUCCESSFUL)
		{
		printf("IPXOpenSocket fails, status = 0x%x, program aborted\n",status);
		return;
		}
/*
 *   Get and display the local network and node.
 *   Also display the socket number just aquired.
 */
	IPXGetInternetworkAddress( (BYTE *)&myaddress);
	printf("IPX based time client started on :\n\t\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);

	printf("\n\t\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);

	printf("\n\t\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n Press any key to abort \n\n");

/*
 *   Prepare an ECB.  This will be used for sending and receiving
 *   the time request packet.  There will be two fragments,
 *   one for the IPX header, and one for the data to send or recieve.
 */
	ipx_ECB.ESRAddress    = (void *) NULL;    /* do not call ESR on completion */
	ipx_ECB.fragmentCount = 2;	
	ipx_ECB.fragmentDescriptor[0].address = &ipx_header;
	ipx_ECB.fragmentDescriptor[0].size    = sizeof(IPXHeader);
	ipx_ECB.fragmentDescriptor[1].address = ipx_data;
	ipx_ECB.fragmentDescriptor[1].size    = 512;

/*
 *   Send a TIME REQUEST to the time server.   Among other
 *   fields, to send an IPX packet we must set the destination
 *   address (net,node,socket) of the time server in the IPX header.
 *
 *   We must also set the immediate address (node) to send the
 *   IPX packet to. This may be the actual address of the
 *   time server or a local bridge who will deliver it to the
 *   correct network.
 *
 *   We can get this immediate address by a call to IPXGetLocalTarget.
 *   It will also return the approximate time to the destination.
 */
	ipx_header.packetType = IPX_PACKET_TYPE;
	memcpy( &ipx_header.destination, server.network, sizeof(IPXAddress));
	ipx_data[0] = TIME_REQUEST;  
																									  
	memcpy(&ipx_ECB.socketNumber,myaddress.socket,2); /* source socket (me)  */
  	status = IPXGetLocalTarget(server.network, &ipx_ECB.immediateAddress,
										&transport_time);

	if (status != SUCCESSFUL)
		{
		printf("IPXGetLocalTarget failed,status =0x%02x, program aborted\n",status);
		return;
		}
	else
		{
		printf("Estimated roundtrip time to server = %d ticks\n",transport_time*2);
		printf("Local time set may be off by approximately this amount\n\n");
		}

	IPXSendPacket(&ipx_ECB);	  /* Away goes the time request */

/*
 *  Check to see if the packet was sent
 *  Successful means sent, but not necessarily recieved
 */
	while(ipx_ECB.inUseFlag != AVAILABLE_FOR_USE )
		{
		IPXRelinquishControl();  /* Give IPX (and others) the CPU     */
		if (kbhit())             /* Cancel the send, exit the program */
			{
			status=IPXCancelEvent(&ipx_ECB);
			printf("Program canceled from keyboard\n");
			printf("IPX Send cancelled, IPX Cancel status =0x%02x\n\n",status);
			return;
			}
		}

	if (ipx_ECB.completionCode != SUCCESSFUL)
		{
		printf("IPXSendPacket failed, program aborted\n");
		printf(" completion code = 0x%02x\n\n",ipx_ECB.completionCode);
		return;
		}

	printf("Time REQUEST sent, waiting for REPLY\n\n");

/*
 *   Listen on my socket for the REPLY, but only listen for
 *   a while.  Since IPX is datagram oriented and not guaranteed
 *   delivery, the REQUEST and/or the REPLY may never have made it.
 *
 *   We use the same ECB used in the send, it should have the
 *   required ECB fields still set.
 */
	i=10;                      /* Wait time in seconds, then we abort */
	valid_reply = FALSE;
	while (!valid_reply)
		{
		memcpy(&ipx_ECB.socketNumber,myaddress.socket,2); /* listen on my socket */
		IPXListenForPacket(&ipx_ECB);                     /* Post the listen ECB */

	/*
	 *   Loop, When something comes in it will reset the inUseFlag.
	 *
	 *   The provider will use one (not necessarily the last) ECB
	 *   posted to fulfill the request, but since we post only one
	 *   we know this is the one chosen.
	 */
		while (ipx_ECB.inUseFlag != AVAILABLE_FOR_USE )
			{
			sleep(1); i--;
			if (i==0)
				{
				printf("Time out waiting for REPLY, program aborted\n\n");
				return;
				}
			IPXRelinquishControl();  /* Give IPX (and others) the CPU       */
			if (kbhit())             /* Cancel the listen, exit the program */
				{
				status=IPXCancelEvent(&ipx_ECB);
				printf("Program canceled from keyboard\n");
				printf("IPX Listen cancelled, IPX Cancel status =0x%02x\n\n",status);
				return;
				}
			} /* end while */

		if (ipx_ECB.completionCode != SUCCESSFUL)
			{
			printf("IPXListenForPacket failed, program aborted\n");
			printf(" completion code = 0x%02x\n\n",ipx_ECB.completionCode);
			return;
			}

	/*
	 *   Check to see if this is really a time reply
	 *   if so, set the system time from that received in the packet.
	 */
		printf("Packet received on client socket from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",ipx_header.source.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",ipx_header.source.node[i]);
		printf("\n\n");

		if (ipx_data[0] != TIME_REPLY)
			printf("\nNon TIME REPLY received on socket, ignored\n\n");
		else
			{
			printf("\nTIME REPLY received on socket, setting system time\n\n");
			valid_reply = TRUE;

			time_zone=malloc(strlen(&ipx_data[1+sizeof(time_t)]) + 8 );
			strcpy(time_zone,"TZ=");
			strcat(time_zone,&ipx_data[1+sizeof(time_t)]);
			putenv(time_zone);         /* set TZ=                */
			tzset();                   /* then make it permanent */
			free(time_zone);

			time_p      = (time_t *)&ipx_data[1];	 /* point to time in data */
			time_of_day = *time_p;                  /* get latest time       */
			stime(&time_of_day);                    /* and set it            */
			}

		} /* end while !valid_reply */

/*	   Get and display the new time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is now %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (time_zone == NULL)
		printf("Time zone is now NULL\n\n");
	else
		printf("Time zone is now %s\n\n",time_zone);

return;			
}  /* end main() */

