
/**/
/*                                                                       */
/*    Common subroutines for routines using                              */
/*    IPX/SPX Netware C Interface for DOS                                */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <time.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include "mrdnxt.h"    /* Missing defines in nxt.h */
#include "timeserv.h"  /* Time server definitions  */

/*
 *   The following routines are defined in this module.
 *   a commented out extern statement for each is included
 *   here to be yanked and used for definition in other modules.
 *
extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);
 *
 */

/*
 *   Global variables used in posting and processing
 *   listening ECBs for SPX routines
 */
ECB  *receivedHead; /* Head of received SPX packets queue        */
ECB  *receivedTail; /* Tail of received SPX packets queue        */


/**/
/*                                                                       */
/*   receiverESR - ESR to post ECB to received queue                     */
/*                                                                       */
/*   Passed   : ECB pointer in ES:SI registers                           */
/*                                                                       */
/*   Modifies : Global variable receivedTail                             */
/*                                                                       */
/*   Returns  :  void                                                    */
/*                                                                       */
/*   CAUTION  : * This routine is executed with interupts disabled,      */
/*               so......... keep it simple and fast.                    */
/*              * This routine MUST be declared huge so that the data    */
/*                segment register will be automatically re-loaded       */
/*                upon entering. This is required because as an          */
/*                interupt routine, we cannot count on any registers     */
/*                assumed setup by the C compiler and we are accesing    */
/*                global data here (receivedTail/Head) which uses ds.    */
/*                                                                       */
/**/

void huge receiverESR(void)
{
	ECB  *ECB_ptr;

/*
 *   Use Borland C++ macro MK_FP to make a far pointer to the ECB
 *   from the ES:SI registers. These registers have been set to point
 *   to the ECB by IPX or AES before calling this ESR. 
 */
	ECB_ptr = (ECB *) MK_FP(_ES,_SI);

/*   Put the completed ECB at the end of the received ECB queue */

	ECB_ptr->linkAddress = 0L;
	receivedTail->linkAddress = (ECB far *)ECB_ptr;
	receivedTail = ECB_ptr;

}	/* end receiverESR */


/**/
/*                                                                       */
/*   InitListenECB - Initialize the Listen ECB necessary to use SPX      */
/*                                                                       */
/*   Passed   : receivedHead - Global variable, used in the ESR          */
/*              receivedTail - Global variable, used in the ESR          */
/*              listenSocket - Socket to post the listen ECBs            */
/*              num_of_ECB   - Number of ECBs to create and post         */
/*              data_size    - Size to allocate for data buffers         */
/*                                                                       */
/*   Requires : listenSocket must be previously opened                   */
/*              ESR receiverESR to handle received data                  */
/*                                                                       */
/*   Modifies : receivedHead, receivedTail                               */
/*                                                                       */
/*   Returns  :  TRUE successfull, FALSE otherwise                       */
/*                                                                       */
/**/

int InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size )
{
/*  */

	BYTE	majorRevisionNumber;		/* returned info from SPXInitialize       */
	BYTE  minorRevisionNumber;
	WORD	maxConnections;
	WORD  availableConnections;

	BYTE        status;     /* scratch status                               */
	int         i;				/* scratch index                                */
	ECB   		*ecbList;   /* ptr to calloc-ed block of ECBs               */
	SPXHeader   *hdrList;   /* ptr to calloc-ed block of SPX headers        */
	BYTE        *dataList;  /* ptr to calloc-ed block of data buffers       */

/*  */

/*   Check if SPX is installed and at least one connection is available    */

	status = SPXInitialize(&majorRevisionNumber,&minorRevisionNumber,
 								  &maxConnections,&availableConnections);
	if (status != SPX_IS_INSTALLED)
		{
		printf("InitListenECB : SPXInitialize failed, SPX is NOT installed\n");
		return FALSE;
		}
	if (availableConnections < 1 )
		{
		printf("InitListenECB : SPXInitialize failed, availableConnections < 1\n");
		return FALSE;
		}

/*   Allocate blocks of ECBs, SPX headers and data buffers all at once */

	ecbList  = calloc( num_of_ECB, sizeof(ECB));
	hdrList  = calloc( num_of_ECB, sizeof(SPXHeader));
	dataList	= calloc( num_of_ECB, data_size);

	if ( (ecbList == NULL) || (hdrList==NULL) || (dataList==NULL) )
		{
		printf("InitListenECB : calloc failed\n");
		return FALSE;
		}

/*
 *   Initialize the Global variables describing the received ECB queue.
 *   We use one dummy ECB as an initial Head and Tail to simplify the
 *   processing. In this way the ESR can add to the end of the queue
 *   without checking if it is first in the queue (gotta make it fast!).
 *   Soooo... The first ECB in the queue is not to be used.
 */
	receivedHead = malloc(sizeof(ECB));
	if ( receivedHead == NULL) 
		{
		printf("InitListenECB : malloc failed\n");
		return FALSE;
		}
	receivedTail = receivedHead;
	receivedHead->linkAddress = NULL;	

/*   Initialize and post to listen each ECB in the block (array) */

	for( i=0; i<num_of_ECB; i++ )
		{
		ecbList[i].ESRAddress    = receiverESR;
		ecbList[i].socketNumber  = listenSocket;
		ecbList[i].fragmentCount = 2;
		ecbList[i].fragmentDescriptor[0].size    = sizeof(SPXHeader);
		ecbList[i].fragmentDescriptor[0].address = (void far *)&hdrList[i];
		ecbList[i].fragmentDescriptor[1].size    = data_size;
		ecbList[i].fragmentDescriptor[1].address = (void far *)&dataList[i];
		SPXListenForSequencedPacket(&ecbList[i]);
		}

	return TRUE;
}	/* end InitListenECB */


/**/
/*                                                                       */
/*   ReceivedSPXPacket - Try to get an SPX Packet from received queue    */
/*                                                                       */
/*   Passed   : receivedHead - Global variable, received queue head      */
/*              receivedTail - Global variable, received queue tail      */
/*              *receivedECB - Ptr to ECB retreived from queue           */
/*                                                                       */
/*   Modifies : receivedHead, receivedTail, receivedECB                  */
/*                                                                       */
/*   Returns  :  TRUE if packet was available, FALSE otherwise           */
/*                                                                       */
/*   CAUTION  : This routine disables/enables interupts                  */
/*                                                                       */
/**/

int ReceivedSPXPacket( ECB **receivedECB )
{

	if (receivedHead->linkAddress != NULL) /* then we received something */
		{
	/*
	 *   We received something, take it off the head of the queue.
	 *   Remember that the first ECB is a dummy, so the real head
	 *   of the list is the next one.
	 */
		*receivedECB = receivedHead->linkAddress;
	/*
	 *   Unlink the ECB from the list while interupts are disabled.
	 *   We do this so that the receiverESR wont mess with the queue
	 *   pointers halfway through the unlinking process.
	 */
		disable(); /* interupts *************************************** */

		receivedHead->linkAddress = (*receivedECB)->linkAddress;
		if ((*receivedECB) == receivedTail) /* then this was the last one */
			receivedTail = receivedHead;  /* so reset the tail          */

		enable(); /* interupts **************************************** */

		return TRUE;
		}
	else
		return FALSE; /* no packet available yet */

}	/* end ReceivedSPXPacket */


/**/
/*                                                                       */
/*   RepostListenECB - Repost a listen ECB that has been processed       */
/*                                                                       */
/*   Passed   : *listenECB   - ECB to repost                             */
/*              listenSocket - Socket to post the listen ECB             */
/*                                                                       */
/*   Requires : listenSocket must be previously opened                   */
/*              ECBs fragment count/descriptor previously initialized    */
/*              ESR receiverESR to handle received data                  */
/*                                                                       */
/*   Modifies : nothing                                                  */
/*                                                                       */
/*   Returns  : void                                                     */
/*                                                                       */
/**/

void RepostListenECB( ECB *listenECB, WORD listenSocket )
{
		listenECB->ESRAddress    = receiverESR;
		listenECB->socketNumber  = listenSocket;
		SPXListenForSequencedPacket(listenECB);

}	/* end RepostListenECB */


/**/
/*                                                                       */
/*   DumpIPXpacket - Dumps an IPX packet using printf                    */
/*                                                                       */
/*   Passed   :  *ipx_header   - Buffer containing IPX Header            */
/*               *userdata_ptr - Buffer containing user data             */
/*               dumpmax       - Max bytes of user data to dump          */
/*                                                                       */
/*   Modifies : nothing                                                  */
/*                                                                       */
/*   Returns  :  void                                                    */
/*                                                                       */
/**/

void DumpIPXpacket( IPXHeader *header, BYTE *userdata_ptr, int dumpmax)
{
	int   length;      /* length of data to dump */
	BYTE  *data_ptr;   /* scratch data pointer   */
	WORD  *operation;  /* RIP or SAP operation   */
	WORD  *serv_type;  /* server type            */
	WORD  socket;      /* socket in ipx header   */
	int   i;           /* for loop index         */

/*   Print the Packet type and header info */

	if (header == NULL) return; /* user tried to pull a fast one */

	switch (header->packetType)
	{
		case 1    : printf("Route Info"); break;
		case 2    : printf("XNS Echo  "); break;
		case 3    : printf("XNS Error "); break;
		case 4    : printf("IPX       "); break;
		case 5    : printf("SPX       "); break;
		case 0x20 : printf("NetBIOS   "); break;
	   default   : printf("Type = %02x ",header->packetType); break;
	};

	printf(" Packet-- ");
	printf("checksum = 0x%x, header length = %d",
			 header->checkSum,IntSwap(header->length));
	printf(" hops = %d\n",header->transportControl);

/*   Now print the data, up to dumpmax number of bytes */

	length = IntSwap(header->length) - 30;  /* subtract header length */
	printf("                  | data length = %d,",length);
	if (length >dumpmax) length = dumpmax;
	printf(" dump length = %d\n",length);

	if (userdata_ptr != NULL)  /* dont dump if pointer is invalid    */
		{
		/*
		 *   Certain sockets are well known and contain formated data.
		 *   Format and dump the data for those special sockets.
		 */
		socket = (WORD) *(WORD *)header->destination.socket;
		switch( socket )
			{
			case 0x5204 : /* Novell SAP protocol socket  */

				printf("                  | Novell SAP ");
				operation = (WORD *)userdata_ptr;

				switch (IntSwap(*operation))	 /* SAP Operation type */
					{
					case 1 :	printf("REQUEST\n");
								break;

					case 2 :	printf("REPLY\n");
								break;

					case 3 :	printf("Get Nearest Server REQUEST\n");
								data_ptr=userdata_ptr;
								data_ptr++; data_ptr++;   /* skip past operation */
								serv_type = (WORD *)data_ptr;
								serv_type = IntSwap(*serv_type);
								data_ptr++; data_ptr++;      /* skip past server type */
 								printf("                  |--type=%02x\n",serv_type);
					         break;

					case 4 :	printf("Get Nearest Server REPLY\n");
								break;

					default: printf("UNKNOWN operation = %x\n",IntSwap(*operation));
								break;
					}

				data_ptr=userdata_ptr;
				data_ptr++; data_ptr++; /* skip past operation */

			/*  Each SAP entry describes a server and is 64 bytes long     */

				while (length >= 64 )   /* dump until we run out of entries */
					{
				/*   Print the server type   */

					serv_type = (WORD *)data_ptr;
					serv_type = IntSwap(*serv_type);
 					printf("                  |--type=%02x ",serv_type);
					data_ptr++; data_ptr++;  /* skip past server type */

				/*   Print server name which is 48 bytes long   */

					for (i=0;i<48;i++) 
						{
						printf("%1c",*data_ptr);
						data_ptr++;
						}

				/*   Print network, node, socket, hops info of this server */

					printf("\n                  |  net=");
					for (i=0;i<4;i++)printf("%02x",*(data_ptr++));
					printf(" node=");
					for (i=0;i<6;i++)printf("%02x",*(data_ptr++));
					printf(" socket=");
					for (i=0;i<2;i++)printf("%02x",*(data_ptr++));
					printf(" hops=");
					for (i=0;i<2;i++)printf("%02x",*(data_ptr++));
					printf("\n");
					length -=64;

					} /* end while */
				break;

			case 0x5304 : /* Novell RIP protocol socket  */

				printf("                  | Novell RIP ");
				operation = (WORD *)userdata_ptr;

				switch (IntSwap(*operation))   /* RIP operation type */
					{
					case 1 :	printf("REQUEST"); break;
					case 2 :	printf("REPLY"); break;
					default: printf("UNKNOWN operation = %x",IntSwap(*operation));
								break;
					}
			/*
			 *   Format and print each 8 byte RIP entry containing the
			 *   network, hops, and time to network
			 */
				printf("\n                  |    Network \tHops\tTime\n");
				data_ptr=userdata_ptr;
				data_ptr++; data_ptr++; /* skip past operation */

				while (length >= 8 )	/* dump until we run out of entries */
					{
					printf("                  |    ");
					for (i=0;i<4;i++)printf("%02x",*(data_ptr++));
					printf("\t");
					for (i=0;i<2;i++)printf("%02x",*(data_ptr++));
					printf("\t");
					for (i=0;i<2;i++)printf("%02x",*(data_ptr++));
					printf("\n");
					length -=8;
					}
				break;

			default: /* Not a well known socket, do a hex dump of the data */

				printf("                  | ");
				for (i=0, data_ptr = userdata_ptr ; i<length; i++, data_ptr++)
					{
					if ( *data_ptr < 16 )
						printf("0%x,",*data_ptr);
					else
						printf("%x,",*data_ptr);
					if (!((i+1)%16)) /* newline every 16 bytes */
						{
						if ((i+1) == length)
							printf("\n");   /* but if it is last line, no spaces please */
						else
							printf("\n                  | ");
						}
					}
				if (i%16) printf("\n"); /* newline if not a multiple of 16 bytes */

			} /* end switch */
		}
	return ;
}	/* end DumpIPXpacket */


/**/
/*                                                                       */
/*   GetIPXAddress - Query user for network,node address                 */
/*                                                                       */
/*   Passed   :  *addr - points to users buffer to receive address       */
/*                                                                       */
/*   Modifies : (*addr).node, (*addr).network                            */
/*                                                                       */
/*   Returns  :  void                                                    */
/*                                                                       */
/**/

void GetIPXAddress( IPXAddress *addr )
{
	int        done;
	char       userstring[80];
	
	done = FALSE;
	while (! done )
		{
		printf("\tEnter 4 byte Network number 0x");
		scanf("%s",userstring);
		fflush(stdin);      /* flush input in case of bad input */
		printf("\n");
		if (strlen(userstring) != 8)
			printf("\tNetwork number must be exactly 8 hex digits\n");
		else
			done = strToHex(userstring,(*addr).network,4);
		}

	done = FALSE;
	while (! done )
		{
		printf("\tEnter 6 byte Node number 0x");
		scanf("%s",userstring);
		fflush(stdin);      /* flush input in case of bad input */
		printf("\n");
		if (strlen(userstring) != 12)
			printf("\tNode number must be exactly 12 hex digits\n");
		else
			done = strToHex(userstring,(*addr).node,6);
		}
}	/* end GetIPXAddress */


/**/
/*                                                                       */
/*   strToHex - Convert an ascii hex string to binary bytes              */
/*                                                                       */
/*   Passed   :  *ascii - ascii string of hex digits                     */
/*               *hex   - area to store converted string                 */
/*               length - number of bytes to convert                     */
/*                                                                       */
/*   Modifies : *hex                                                     */
/*                                                                       */
/*   Returns  :  TRUE if converted, FALSE otherwise                      */
/*                                                                       */
/**/

int strToHex(char *ascii, BYTE *hex, int length)
{
	int i,j;
	char tmp_char;

	for (i=0,j=0; i<2*length; i+=2,j++)
		{
		if (!isxdigit(tmp_char=toupper(ascii[i])) )
			{
			printf("Illegal hex digit \n");
			return FALSE;
			}
		else
			hex[j] = ((tmp_char>'9') ? (tmp_char-0x37) : (tmp_char-0x30)) << 4;

		if (!isxdigit(tmp_char=toupper(ascii[i+1])) )
			{
			printf("Illegal hex digit \n");
			return FALSE;
			}
		else
			hex[j] += (tmp_char > '9') ? (tmp_char-0x37) : (tmp_char-0x30);
		}

	return TRUE;
}	/* end strToHex */


/**/
/*                                                                       */
/*    SendSPXPacket - Sends an SPX data packet                           */
/*                                                                       */
/*   Passed   :  connectionID  - Connection ID to send data on           */
/*               *data         - Pointer to users data to send           */
/*                datalen      - Length of data to send                  */
/*                                                                       */
/*   Modifies : nothing                                                  */
/*                                                                       */
/*   Returns :  TRUE if sent successfully, FALSE otherwise               */
/*                                                                       */
/**/

int SendSPXPacket( WORD connectionID, void *data, WORD datalen)
{
	ECB	    sendECB;
	SPXHeader SendHDR;

/*
 *   Build the ECB and SPX header. Since these are stack variables
 *   they will be removed after this routine returns, hence no
 *   ESR is needed AND we must wait for completion before returning.
 */
	sendECB.ESRAddress    = NULL;
	sendECB.fragmentCount = 2;
	sendECB.fragmentDescriptor[0].size    = sizeof(SPXHeader);
	sendECB.fragmentDescriptor[0].address = (void far *)&SendHDR;
	sendECB.fragmentDescriptor[1].size    = datalen;
	sendECB.fragmentDescriptor[1].address = data;

	SendHDR.connectionControl = SPX_END_OF_MESSAGE;	/* assume this for user  */
	SendHDR.dataStreamType    = 0x00;					/* not end of connection */

 	SPXSendSequencedPacket(connectionID, &sendECB);

/*   Wait until SPX is done with all buffers */

 	while (sendECB.inUseFlag != AVAILABLE_FOR_USE )
		IPXRelinquishControl();

 	if (sendECB.completionCode != SUCCESSFUL )
		{
 		printf("\nSendSPXPacket : SPXSendSequencedPacket failed,\n");
		printf("                  completionCode = %02X\n",sendECB.completionCode);
 		return FALSE;
		}

	return TRUE;
}	/* end SendSPXPacket */


/**/
/*                                                                       */
/*   EstablishConnection -  Establish an SPX connection                  */
/*                                                                       */
/*   Passed   :  *destination  - Destination Address of connection       */
/*               *connectionID - Ptr to connection Id returned to user   */
/*               mySocket      - Socket for connection                   */
/*                                                                       */
/*   Requires :  At least 2 listen ECBs must have been posted before     */
/*               calling this routine. One for the confirmation packet,  */
/*               and one for the watchdog process which keeps track      */
/*               of the connection to see when it goes down.             */
/*               mySocket must be previously opened by IPXOpenSocket.    */
/*                                                                       */
/*   Modifies : connectionID set to provider supplied ID for connection  */
/*                                                                       */
/*   Returns  :  True if connection established, FALSE otherwise         */
/*                                                                       */
/**/

int   EstablishConnection(	IPXAddress *destination,	WORD *connectionID,
									WORD        mySocket)
{
	ECB			connectECB;
	SPXHeader 	connectHDR;
	BYTE		 	RetryCount=0x00;            /* Use SPXs internal count       */
	BYTE			WatchDog  =ENABLE_WATCHDOG; /* Watch connection for failures */
	WORD			status;							 /* scratch status                */

/*
 *   Build the ECB. Since the ECB is a stack variable it will be
 *   removed after this routine returns, hence no ESR is needed AND we
 *   must wait for completion before returning.
 */
	connectECB.ESRAddress    = NULL;
	connectECB.socketNumber  = mySocket;
	connectECB.fragmentCount = 1;
	connectECB.fragmentDescriptor[0].address = (void far *)&connectHDR;
	connectECB.fragmentDescriptor[0].size    = sizeof(SPXHeader);
	memmove(&connectHDR.destination,destination,sizeof(IPXAddress));

/*   Initiate the connection, then wait for  completion */

	status = SPXEstablishConnection(RetryCount, WatchDog, connectionID,
											  &connectECB);
	if (status != SPX_CONNECTION_STARTED)
		{
		printf("\nEstablishConnection : SPXEstablishConnection failed,\n");
		printf("SPX_CONNECTION not STARTED, return value =  %02X\n",status);
		return FALSE;
		}

/*   Wait for actual completion of connection process */

 	while (connectECB.inUseFlag != AVAILABLE_FOR_USE) 
 		IPXRelinquishControl();

/*   Connection completed, But was it successful ? */

	if (connectECB.completionCode != SUCCESSFUL)
		{
		printf("\nEstablishConnection : SPXEstablishConnection failed,\n");
		printf("                        completionCode = %02X\n",
				 connectECB.completionCode);
		return FALSE;
		}

	return TRUE;
}  /* end EstablishConnection */


/**/
/*                                                                       */
/*   TerminateConnection - Terminates an SPX connection.                 */
/*                                                                       */
/*   Passed  :  connectionID - Connection Id to terminate                */
/*                                                                       */
/*   Modifies : nothing                                                  */
/*                                                                       */
/*   Returns :  TRUE if connection terminated normally, FALSE otherwise  */
/*                                                                       */
/**/

int TerminateConnection( WORD connectionID )
{
	ECB		 terminateECB;
	SPXHeader terminateHDR;

/*
 *   Build the ECB. Since the ECB is a stack variable it will be
 *   removed after this routine returns, hence no ESR is needed AND we
 *   must wait for completion before returning.
 */
	terminateECB.ESRAddress    = NULL;
	terminateECB.fragmentCount = 1;
	terminateECB.fragmentDescriptor[0].address = (void far *)&terminateHDR;
	terminateECB.fragmentDescriptor[0].size    = sizeof(SPXHeader);

	SPXTerminateConnection( connectionID, &terminateECB);

/*   Wait until SPX finishes termination negotion with its peer */

	while ( terminateECB.inUseFlag != AVAILABLE_FOR_USE )
		IPXRelinquishControl();

/*   Check completion code to see if termination process went well */

	if (terminateECB.completionCode != SUCCESSFUL )
		{
		printf("\nTerminateConnection : SPXTerminateConnection failed,\n");
		printf("                        completionCode = %02X\n",
				 terminateECB.completionCode);
		return FALSE;
		}

	return TRUE;
}	/* end TerminateConnection */

