#include <nwnet.h>
#include <stdio.h>
#include <string.h>
#include <alloc.h>

#define FALSE 0
#define TRUE 1

// *****************************
// Example code for List Objects
// *****************************
void main()
{ int               err,i;
  NWDSContextHandle nContext;
  Buf_T             *ResBuf;
  int32             iterHandle=NO_MORE_ITERATIONS;
  char              objName[MAX_DN_CHARS];
  uint32            totObjects,totAttrs;
  Object_Info_T     objectInfo;

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("Could not create context\n");
      goto EndProc;
    }

  // Set context and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"C=US");
  if(err<0)
    { printf("Error in NWDSSetContext: <%d>\n", err);
      goto EndProc;
    }

  // Allocate space for result buffer and check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ResBuf);
  if(err<0)
    { printf("Error in NWDSAllocBuf: <%d>\n", err);
      goto EndProc;
    }

  // Call to NWDSList and check for errors.  iterHandle must be initialized
  // to -1.  NWDSList should be called until iterHandle is NO_MORE_ITERATIONS.
  // In the case that the result spans multiple output buffers, iterHandle
  // will be something other than NO_MORE_ITERATIONS.  Do not change the
  // value returned.  Just send it back with the next call to NWDSList.
  iterHandle = NO_MORE_ITERATIONS;
  do
    { err = NWDSList(nContext, "O=Novell", &iterHandle, ResBuf);
      if(err<0)
	{ printf("Error in NWDSList: <%d>\n", err);
	  goto EndProc;
	}

      // Pull information from output buffer.  Check for errors.
      // First get the number of objects in the buffer.  Check for errors.
      err = NWDSGetObjectCount(nContext, ResBuf, &totObjects);
      if(err<0)
	{ printf("Error in NWDSGetObjectCount: <%d>\n", err);
	  goto EndProc;
	}

      for(i=0;i<totObjects;i++)
	{
	  // Pull object name, total number of attributes associated with
	  // the object, and an object info structure from buffer.  Check
	  // for errors.
	  err = NWDSGetObjectName(nContext, ResBuf, objName, &totAttrs,
							    &objectInfo);
	  if(err<0)
	    { printf("Error in NWDSGetObjectName: <%d>\n", err);
	      goto EndProc;
	    }
	  printf("Object found --> %s\n", objName);
	}
    } while(iterHandle != -1);

  // Clean up buffers and context.  These functions do not return anything.
EndProc:
  NWDSFreeBuf(ResBuf);
  NWDSFreeContext(nContext);
}
