#include <nwnet.h>
#include <stdio.h>

#define FALSE 0
#define TRUE 1

// ***********************
// Example code for Search
// ***********************
void main()
{ NWDSContextHandle nContext;
  int               err,j,k,l;
  Buf_T             *FiltBuf, *NameBuf, *ResBuf;
  int32             iterHandle=-1L;
  Filter_Cursor_T   *cur;
  uint32            totObj,totAttr,totVal,synID;
  char              ObjName[MAX_DN_CHARS+1];
  Object_Info_T     objInfo;

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("ERROR: Could not create context\n");
      goto EndProc;
    }

  // Set context and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"C=US");
  if(err<0)
    { printf("ERROR: in NWDSSetContext: <%d>\n",err);
      goto EndProc;
    }

  // Allocate space for NameBuf, FiltBuf, and Result buffers.  Check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN,&NameBuf);
  if(err<0)
    { printf("ERROR: in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN,&FiltBuf);
  if(err<0)
    { printf("ERROR: in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN,&ResBuf);
  if(err<0)
    { printf("ERROR: in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }

  // Initialize NameBuf and Filter buffers.  Check for errors.  The output
  // buffer does not need to be initialized.

  // The NameBuf buffer will be initialized to the DSV_SEARCH operation.
  err = NWDSInitBuf(nContext, DSV_SEARCH, NameBuf);
  if(err<0)
    { printf("ERROR: in NWDSInitBuf: <%d>\n",err);
      goto EndProc;
    }

  // The FiltBuf buffer will be initialized to the DSV_SEARCH_FILTER
  // operation.
  err = NWDSInitBuf(nContext, DSV_SEARCH_FILTER, FiltBuf);
  if(err<0)
    { printf("ERROR: in NWDSInitBuf: <%d>\n",err);
      goto EndProc;
    }

  // Create search filterexpression tree.  Check for errors.
  err = NWDSAllocFilter(&cur);
  if(err<0)
    { printf("ERROR: in NWDSAllocFilter: <%d>\n",err);
      goto EndProc;
    }

  // Fill the FiltBuf with the tokens that represent the search expression.
  // Check for errors.  The expression represented by these tokens is
  //                 Object Class="User"
  // If the value and syntax parameters are not defined for a particular
  // token then NULL can be sent in its place.  These parameters are ignored
  // in such cases.
  err = NWDSAddFilterToken(cur,FTOK_ANAME,"Object Class",SYN_CLASS_NAME);
  if(err<0)
    { printf("ERROR: in NWDSAddFilterToken: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAddFilterToken(cur,FTOK_EQ,NULL,(uint32)0);
  if(err<0)
    { printf("ERROR: in NWDSAddFilterToken: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAddFilterToken(cur,FTOK_AVAL,"User",SYN_CLASS_NAME);
  if(err<0)
    { printf("ERROR: in NWDSAddFilterToken: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAddFilterToken(cur,FTOK_END,NULL,(uint32)0);
  if(err<0)
    { printf("ERROR: in NWDSAddFilterToken: <%d>\n",err);
      goto EndProc;
    }

  // Put the completed search expression into the FiltBuf buffer.  Check for
  // errors.
  err = NWDSPutFilter(nContext,FiltBuf,cur,NULL);
  if(err<0)
    { printf("ERROR: in NWDSPutFilter: <%d>\n",err);
      goto EndProc;
    }

  // The only attribute that will be searched for is "Surname".  This
  // attribute is stored in the NameBuf buffer.  Check for errors.
  err = NWDSPutAttrName(nContext, NameBuf, "Surname");
  if(err<0)
    { printf("ERROR: in NWDSPutAttrName: <%d>\n",err);
      goto EndProc;
    }

  // Call to NWDSSearch.  Check for errors.  The parameters are set as
  // follows:
  // objectName = O=Novell.
  // scope = The subtree from O=Novell.
  // alias = FALSE - aliases are not dereferenced.
  // type = 1 - both attribute names and values are to be returned.
  // allAttrs = FALSE - Attributes to be retrieved are in the NameBuf buffer.
  // iterHandle is initialized to NO_MORE_ITERATIONS.  NWDSSearch should be
  // called until iterHandle is NO_MORE_ITERATIONS.  Any other value means
  // there is more data to be retrieved.
  iterHandle = NO_MORE_ITERATIONS;
  do
    { err = NWDSSearch(nContext,"O=Novell",DS_SEARCH_SUBTREE,FALSE,FiltBuf,
	  (int32)1,FALSE,NameBuf,&iterHandle,0,(int32 *)NULL,ResBuf);
      if(err<0)
	{ printf("ERROR: in NWDSSearch: <%d>\n",err);
	  goto EndProc;
	}

      // Pull the information out of the ResBuf buffer.  Remember to pull
      // out everything that is in the buffer, whether all of it is wanted
      // or not.  If both attribute names and values are requested, both
      // names and values must be pulled out.  Failure to pull the values
      // will make it impossible to pull out the rest of the attribute
      // names.

      // Get the count of objects in the ResBuf buffer.  Check for errors.
      err = NWDSGetObjectCount(nContext,ResBuf,&totObj);
      if(err<0)
	{ printf("ERROR: in NWDSGetObjectCount: <%d>\n",err);
	  goto EndProc;
	}

      // Pull out the object information.  Check for errors.
      for(j=0;j<totObj;j++)
      { err = NWDSGetObjectName(nContext,ResBuf,ObjName,&totAttr,&objInfo);
	if(err<0)
	  { printf("ERROR: in NWDSGetObjectName: <%d>\n",err);
	    goto EndProc;
	  }
	printf("\nObject Name: %s\n",ObjName);

	// Pull information about each object from ResBuf buffer.  Check
	// for errors.
	for(k=0;k<totAttr;k++)
	  { err = NWDSGetAttrName(nContext,ResBuf,ObjName,&totVal,&synID);
	    if(err<0)
	      { printf("ERROR: in NWDSGetAttrName: <%d>\n",err);
		goto EndProc;
	      }
	    printf("     Attr Name -> %s\n",ObjName);

	    // Pull attribute values from ResBuf buffer.  Check for errors.
	    for(l=0;l<totAttr;l++)
	      { err = NWDSGetAttrVal(nContext, ResBuf, synID, ObjName);
		if(err<0)
		  { printf("ERROR: in NWDSGetAttrVal: <%d>\n",err);
		    goto EndProc;
		  }
		printf("          Attr Val -> %s\n",ObjName);
	      }
	  }
      }
    } while(iterHandle != NO_MORE_ITERATIONS);

  // Clean up buffers and context.  These functions do not return anything.
EndProc:
  NWDSFreeContext(nContext);
  NWDSFreeBuf(NameBuf);
  NWDSFreeBuf(FiltBuf);
  NWDSFreeBuf(ResBuf);
}
