
/**/
/*                                                                       */
/*	          SPX Based Time Query Client                               */
/*                                                                       */
/*   This routine will query the user for network and node address of    */
/*   any SPX time server. It will attempt a connection with that server. */
/*   If successful, it It will then send a time request to contained     */
/*   with SPX data.  When the response is received, the routine will     */
/*   set the local and then attempt to disconnect gracefully.            */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"    /* Missing defines in nxt.h */
#include "timeserv.h"  /* Time server definitions  */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main() 
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	time_t     *time_p;       /* used for conversion                        */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* ptr to TZ= string                          */

	WORD        serverSocket;   /* Temp WORD area for server socket         */
	WORD        mySocket;       /* Temp WORD area for my socket             */
	WORD        connectionID;   /* current connection ID                    */
	int 	      connection_open; /* Boolean : is there a connection open ?  */

	IPXAddress  myaddress;    /* My network,node,socket                     */
	IPXAddress  server;       /* Server network,node,socket                 */

	ECB         sendECB;       /* ECB used to send time request             */
	SPXHeader   sendHdr;       /* SPX header used for time request          */
	BYTE        sendData[512]; /* SPX data containing time request          */

	ECB       *processECB;   /* Current ECB retreived from queue            */
	SPXHeader *processHdr;   /* Current SPX header                          */
	BYTE      *processData;  /* Current SPX data                            */

	int         status;       /* scratch status                             */
	int         i;            /* scratch index                              */

/*  */

/*   Get and display the old time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("\nThe time was %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (*time_zone == NULL)
		printf("Time zone was NULL\n");
	else
		printf("Time zone was %s\n\n",time_zone);

/*   Initialize the IPX function library   */

	if ((status=IPXInitialize()) != SUCCESSFUL )
		{
		printf("IPXInitialize fails, status = 0x%x, program aborted\n",status);
		return;
		}
/*
 *   Ask the user which timeserver he wants to use. We do this because
 *   advertising/geting server only works from a workstation perspective
 *   if there is an intermediate router or if the workstation implements
 *   a complete SAP handler, which is not the case in this simple example.
 */
	printf("Please enter the timeserver address\n\n");
	GetIPXAddress(&server);
	serverSocket=TIME_SERVER_SOCKET;			 /* server socket is well known */
	memcpy(server.socket,&serverSocket,2);

/*
 *   Open a dynamic socket for the client to use by specifying 0.
 *   The actual socket allocated is returned to us. We open this socket
 *   as SHORT_LIVED, it will close automatically on program termination.
 */
	memset(myaddress.socket,0,2);
	if ((status=IPXOpenSocket(myaddress.socket,SHORT_LIVED))!=SUCCESSFUL)
		{
		printf("IPXOpenSocket fails, status = 0x%x, program aborted\n",status);
		return;
		}
	memcpy(&mySocket,myaddress.socket,2); /* save in WORD form */

/*
 *   Get and display the local network and node.
 *   Also display the socket number just aquired.
 */
	IPXGetInternetworkAddress( (BYTE *)&myaddress);
	printf("SPX based time client started on :\n\t\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);

	printf("\n\t\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);

	printf("\n\t\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n Press any key to abort \n\n");

/*
 *   Create and post a list of ECBs and data buffers used for listening.
 *
 *   These are posted to my opened socket.  Since SPX/IPX does not
 *   have its own internal buffers, we must provide ECBs and buffers
 *   for it to use.  Create and post 5, this gives one for the
 *   connect confirm, one for the watchdog and 3 for normal data.
 *   Make each associated data buffer 512 bytes long, should be enough.
 */
	InitListenECB(mySocket, 5, 512);

/*   Open a connection to the time server   */

	if ( ! EstablishConnection( &server, &connectionID, mySocket) )
		{
		printf("Could not establish connection with server, program aborted\n\n");
		return;
		}
	else
		{
		printf("Established connection with server, connection ID = 0x%04x\n\n",
				connectionID);
		connection_open = TRUE;
		}

/*
 *   Send a TIME REQUEST to the time server. 
 */

	sendECB.ESRAddress    = NULL;
	sendECB.fragmentCount = 2;
	sendECB.fragmentDescriptor[0].size    = sizeof(SPXHeader);
	sendECB.fragmentDescriptor[0].address = &sendHdr;
	sendECB.fragmentDescriptor[1].size    = 512;
	sendECB.fragmentDescriptor[1].address = sendData;
	sendHdr.connectionControl = 0x00;
	sendHdr.dataStreamType    = 0x00;

	sendData[0] = TIME_REQUEST;   /* time request operation   */

	SPXSendSequencedPacket(connectionID, &sendECB);
	while (sendECB.inUseFlag != AVAILABLE_FOR_USE)
		{
		IPXRelinquishControl();
		if (kbhit())     
			{
			printf("\nProgram canceled from keyboard\n");
			return;
			}
		}
	if (sendECB.completionCode != SUCCESSFUL)
		printf("TIME REQUEST FAILED, completionCode=0x%02X\n\n",
				sendECB.completionCode);
	else
		printf("Time REQUEST sent, waiting for REPLY\n\n");

/*
 *   Process all SPX Packets (ECB) received on this connection.
 *
 *   Listen ECBs were preposted before the connection was made.
 *   We will take the first TIME_REPLY received and then
 *   terminate the connection.  There may be other packets or
 *   completions of the preposted listen ECBs which we must also
 *   handle in this loop until we receive a valid reply.
 */
	while ( connection_open )
		{
		while ( ! ReceivedSPXPacket(&processECB) )
			{
			IPXRelinquishControl();  /* Give up the CPU for a while     */
			if (kbhit())             /* User gives up, exit the program */
				{
				TerminateConnection(connectionID);
				printf("Program canceled from keyboard\n");
				return;
				}
			}
	/*
	 *   We have an SPX packet (ECB) to process.  It could be data,
	 *   it could be a disconnect. Check for and process each possibility.
	 */
		processHdr  = processECB->fragmentDescriptor[0].address;
		processData = processECB->fragmentDescriptor[1].address;
		printf("SPX Data Packet received from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",processHdr->source.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",processHdr->source.node[i]);
		printf("\n\n");

	/*
	 *   The completion code of the ECB tells why this
	 *   ECB returned from listening
	 */
		switch ( processECB->completionCode )
			{
			case SPX_CONNECTION_FAILED :  /* Watchdog discovered failure */
				printf("Watchdog detected connection failed\n\n");
				connection_open =FALSE;
				break;
			case SPX_PACKET_OVERFLOW   :  /* Too big to be a time reply */
				printf("SPX Packet Overflow detected, ignored\n\n");
				break;
			case SPX_SOCKET_NOT_OPENED : /* Oops, my mistake  */
			case SPX_SOCKET_CLOSED     :
				printf("Socket problems, connection terminated\n\n");
				connection_open =FALSE;
				break;

			case SPX_PACKET_SUCCESSFUL : /* Some real data to play with   */
				/*
				 *   Check to see if this is really a time reply,
				 *   If so, extract the time and set ours. The packet could
				 *   be a disconnect or system packet.
				 */
				if (processHdr->dataStreamType == TERMINATE_CONNECTION)
					{
					printf("Peer terminated connection\n\n");
					connection_open = FALSE;
					break;
					}

				if (processHdr->connectionControl & SPX_SYSTEM_PACKET)
					{
					printf("System packet received, ignored\n\n");
					break;
					}

			  	if (processData[0] != TIME_REPLY)
					{
					printf("\nNon TIME REPLY packet received on socket, ignored\n\n");
					break;
					}
				else
				/*   This seems to be a valid reply, set the system time
				 *   and timezone from the values received in the packet.
				 */
					{
					printf("\nValid TIME REPLY received, processing now.\n");

					time_zone=malloc(strlen(&processData[1+sizeof(time_t)]) + 8 );
					strcpy(time_zone,"TZ=");
					strcat(time_zone,&processData[1+sizeof(time_t)]);
					putenv(time_zone);     /* setting the TZ environment var */
					tzset();               /* and making it permanent        */
					free(time_zone);

					time_p      = (time_t *)&processData[1]; /* point to time  */
					time_of_day = *time_p;                  /* get latest time */
					stime(&time_of_day);                    /* and set it      */

					} /* end valid TIME REPLY */
			/*
			 *   Now that we have received and processed a valid
			 *   time reply, we can close down the connection gracefully.
			 */
				TerminateConnection( connectionID );
				connection_open = FALSE;
				break;

			default : /* Unknown SPX completion code */

				printf("Unknown SPX completion code = 0x%02x\n\n",
							processECB->completionCode);
				break;

			}	/* end switch processECB->completionCode */

		/*
		 *   Repost the SPX packet, we are done with it.
		 *   Go back and look for more packets
		 */
		   if (connection_open)	/* dont bother reposting if we are done */
		  		RepostListenECB(processECB, serverSocket);

		} /* end while connection_open */

/*	   Get and display the new time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is now %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (time_zone == NULL)
		printf("Time zone is now NULL\n\n");
	else
		printf("Time zone is now %s\n\n",time_zone);

return;			
}  /* end main() */

