
/**/
/*                                                                       */
/*                    SPX Based Time Server                              */
/*    		                                                                 */
/*   This routine will act as a time server.  Connection                 */
/*   establishment and termination with a client will be handled         */
/*   on its well known socket. It will accept time requests only         */
/*   on an established connection. It will reply to that request         */
/*   with the current time returned in the SPX data portion.             */
/*   All other connection attempts will be rejected until the            */
/*   current connection is terminated.                                   */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"    /* Missing defines in nxt.h */
#include "timeserv.h"  /* Time server definitions  */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main()
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* TZ environment variable                    */

	IPXAddress  myaddress;    /* My network,node,socket                     */
	WORD        serverSocket; /* Temp holding area for server socket        */

	ECB         connectECB;      /* Used for Listen for connection          */
	int         connection_open; /* Boolean, connection currently open ?    */
	WORD        connectionID;    /* current connection ID                   */

	int         status;        /* scratch status                            */
	int         i;             /* scratch index                             */

	ECB         sendECB;       /* ECB used to send time reply               */
	SPXHeader   sendHdr;       /* SPX header used for time reply            */
	BYTE        sendData[512]; /* SPX data containing time reply            */

	ECB       *processECB;   /* Current ECB retreived from queue            */
	SPXHeader *processHdr;   /* Current SPX header                          */
	BYTE      *processData;  /* Current SPX data                            */

/*  */

/*   Get and display the time   */

	time_zone = getenv("TZ");
	if (time_zone == NULL)
		{
		printf("\nTime zone variable (TZ=) must be set before running server\n");
		printf("Time server aborted\n\n");
		return;
		}
	else
		printf("\nTime zone is %s\n",time_zone);

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is %s\n", asctime(time_block) );

/*   Initialize the IPX function library, we may use some of its functions */

	if ((status=IPXInitialize()) != SUCCESSFUL )
		{
		printf("IPXInitialize fails, status = 0x%x, program aborted\n\n",status);
		return;
		}

/*
 *   Open a well known socket for the timeserver to use.
 *   We open this socket as SHORT_LIVED, it will close on program termination.
 */
	serverSocket=TIME_SERVER_SOCKET;
	memcpy(myaddress.socket,&serverSocket,2);
	if ((status=IPXOpenSocket(myaddress.socket, SHORT_LIVED )) != SUCCESSFUL)
		{
		printf("IPXOpenSocket fails, status = 0x%x, program aborted\n\n",status);
		return;
		}

/*
 *   Get and display the local network and node and socket.
 */
	IPXGetInternetworkAddress( (BYTE *)&myaddress);

	printf("SPX based timeserver started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*
 *   Create and post a list of ECBs and data buffers used for listening.
 *
 *   These are posted to my opened socket.  Since SPX/IPX does not
 *   have its own internal buffers, we must provide ECBs and buffers
 *   for it to use.  Create and post 5, this gives one for the
 *   connect confirm, one for the watchdog and 3 for normal data.
 *   Make each associated data buffer 512 bytes long, should be enough.
 */
	InitListenECB(serverSocket, 5, 512);

/*
 *                 Main Processing Loop
 *
 *   Listen for an incoming SPX connection. An ECB is passed and
 *   this particular ECB will be used to inform me of success.
 *
 *   We pass zero as the retry count which instructs SPX to use
 *   its own internal value and enable watchdog which will inform
 *   us of connection failure during conversation. 
 */
	connection_open = FALSE;
	while ( !kbhit() )       /* allow user to abort program */
		{
		connectECB.fragmentCount= 0;
		connectECB.ESRAddress   = NULL;
		connectECB.socketNumber = serverSocket;
		SPXListenForConnection( 0, ENABLE_WATCHDOG, &connectECB);
		while(connectECB.inUseFlag != AVAILABLE_FOR_USE )
			{
			IPXRelinquishControl();  /* Give IPX (and others) the CPU     */
			if (kbhit())             /* Cancel the listen, exit the program */
				{
				IPXCancelEvent(&connectECB);
				printf("\nProgram canceled from keyboard\n");
				printf("Cancelled SPXListenForConnection\n\n");
				return;
				}
			}
	/*
	 *   The connection ID is recorded in the first two bytes of
	 *   the connectECBs IPXWorkspace field, save it for use.
	 */
		connectionID =(WORD) *( (WORD *)connectECB.IPXWorkspace);
		printf("SPX connection established, connection ID = 0x%04x\n",connectionID);
	 	connection_open = TRUE;

	/*
	 *   We have a connection.   Process any and all received data
	 *   packets received from this connection.
	 *
	 *   The Event Service routine associated with the listen ECBs will
	 *   post each ECB to my received packet queue when they are received.
	 *   They must be queued like this to maintain the sequence
	 *   which they arrived.  I will process each one and repost it
	 *   for listening when I am done.
	 */
		while ( connection_open )  /* process all incoming packets */
			{
			while ( ! ReceivedSPXPacket(&processECB) )
				{
				IPXRelinquishControl();  /* Give up the CPU for a while     */
				if (kbhit())             /* User gives up, exit the program */
					{
					TerminateConnection(connectionID);
					printf("Program canceled from keyboard\n");
					return;
					}
				}
		/*
		 *   We have an SPX packet (ECB) to process.  It could be data,
		 *   it could be a disconnect. Check for and process each possibility.
		 */
			processHdr  = processECB->fragmentDescriptor[0].address;
			processData = processECB->fragmentDescriptor[1].address;

			printf("SPX Data Packet received from :\n\tNetwork number = 0x");
			for (i=0;i<4;i++)printf("%02x",processHdr->source.network[i]);
			printf("\n\tNode number    = 0x"); 
			for (i=0;i<6;i++)printf("%02x",processHdr->source.node[i]);
			printf("\n\n");

		/*
		 *   The completion code of the ECB tells why this
		 *   ECB returned from listening
		 */
			switch ( processECB->completionCode )
				{
				case SPX_CONNECTION_FAILED :  /* Watchdog discovered failure */
					printf("Watchdog detected connection failed\n\n");
					connection_open =FALSE;
					break;

				case SPX_PACKET_OVERFLOW   :  /* Too big to be a time request */
					printf("SPX Packet Overflow detected, ignored\n\n");
					break;

				case SPX_SOCKET_NOT_OPENED : /* Oops, my mistake  */
				case SPX_SOCKET_CLOSED     :
					printf("Socket problems, connection terminated\n\n");
					connection_open =FALSE;
					break;

				case SPX_PACKET_SUCCESSFUL : /* Some real data to play with   */
					/*
					 *   Check to see if this is really a time request,
					 *   If so, build a response and send it. It could
					 *   be a disconnect or system packet.
					 */
					if (processHdr->dataStreamType == TERMINATE_CONNECTION)
						{
						printf("Peer terminated connection\n\n");
						connection_open = FALSE;
						break;
						}

					if (processHdr->connectionControl & SPX_SYSTEM_PACKET)
						{
						printf("System packet received, ignored\n\n");
						break;
						}

					if (processData[0] != TIME_REQUEST)
						{
						printf("\nNon TIME REQUEST received on socket, ignored\n\n");
						break;
						}
					else
					/*   This seems to be a valid request, build a time packet and
					 *   send it back to the client.
					 */
						{
						printf("\nValid TIME REQUEST received, processing now.\n");
		
						sendECB.ESRAddress    = NULL;
						sendECB.fragmentCount = 2;
						sendECB.fragmentDescriptor[0].size    = sizeof(SPXHeader);
						sendECB.fragmentDescriptor[0].address = &sendHdr;
						sendECB.fragmentDescriptor[1].size    = 512;
						sendECB.fragmentDescriptor[1].address = sendData;

						sendHdr.connectionControl = 0x00;
						sendHdr.dataStreamType    = 0x00;

					/*
					 *   Get the time into the data area at the last possible moment
					 *   and then send the time packet on its merry way.
					 */	
						sendData[0] = TIME_REPLY;      /* time reply operation   */
						time_of_day = time(NULL);      /* get latest time        */
						time_zone = getenv("TZ");		 /* and time zone          */
						memcpy(&sendData[1], &time_of_day, sizeof(time_t));
						strcpy(&sendData[1+sizeof(time_t)],time_zone);

						SPXSendSequencedPacket(connectionID, &sendECB);
						while (sendECB.inUseFlag != AVAILABLE_FOR_USE)
							{
							IPXRelinquishControl();
							if (kbhit())     
								{
								printf("\nProgram canceled from keyboard\n");
								return;
								}
							}

						if (sendECB.completionCode != SUCCESSFUL)
							printf("TIME REPLY FAILED, completionCode=0x%02X\n\n",
									sendECB.completionCode);
						else
							printf("TIME REPLY sent successfully.\n\n");

						} /* end valid TIME REQUEST */
					break;

				default : /* Unknown SPX completion code */

					printf("Unknown SPX completion code = 0x%02x\n\n",
								processECB->completionCode);
					break;

				}	/* end switch processECB->completionCode */

			/*
			 *   Repost the SPX packet, we are done with it.
			 *   Go back and look for more packets
			 */
			  	RepostListenECB(processECB, serverSocket);

			} /* end while connection_open */
	/*
	 *   The connection is gone. Flush all leftover received ECBs
	 *   from the queue and repost them to listen in preparation
	 *   to receive another connection
	 */
		 while ( ReceivedSPXPacket(processECB) )
		 	RepostListenECB(processECB, serverSocket);

		} /* end while !kbhit() */

	return;
}  /* end main() */
