/****************************************************************************
 *                                                                          *
 *             Copyright 1992  Novell, Inc.  All Rights Reserved.           *
 *                                                                          *
 * THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND        *
 * TREATIES.  THIS WORK MAY BE USED, COPIED, DISTRIBUTED, REVISED, MODIFIED *
 * TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED   *
 * RECAST, TRANSFORMED, OR ADAPTED  SO LONG AS THIS COPYRIGHT NOTICE IS		*
 * MAINTAINED.  ANY OTHER USE OR USE WITHOUT MAINTAINING THIS COPYRIGHT		*
 * COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY. 			*
 *                                                                          *
 ***************************************************************************/

/**/
/*                                                                       */
/*            Multi Protocol Time Server Client using SPX                */
/*		                                                                  */
/*   This is a time server client.  It works with SPX only using the     */
/*   SPX APIs.  After determining the service address it establishes a	  */
/*   connection with the time and makes a time request, wait for the 	  */
/*   reply, display the time and then exit.							  */
/*                                                                       */
/**/


/**** Include Files ****/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <nxt.h>
#include <nit.h>
/****************************************************************************/


/**** Environment Configuration ****/

#ifndef TRUE
	#define TRUE	1
	#define	FALSE	0
#endif
#ifndef	nilp
	#define	nilp(s)		((s *)0)
#endif

#define LISTEN_ECB_COUNT	4
/****************************************************************************/


/**** Local Prototypes ****/

int SPXParseAddress(char *text, struct IPXAddress *spxAddr);
/****************************************************************************/


/**** Time Server controls ****/
/* Function Numbers */
#define TS_TIME_REQUEST		1			/* Function */
#define 	TS_TR_VER			1		/* Function Version */
#define TS_TIME_SYNC		2
#define 	TS_TY_VER			1
#define TS_TIME_SET			3
#define 	TS_TS_VER			1

/* Completion Codes */
#define TS_INVALID_REQUEST		-1
#define TS_UNSUPORTED_VERSION	-2
#define TS_REQUEST_COMPLETE	 	 0

typedef struct timeServerHeader {	/* Size should be multiple of DWORD */
	long int	function;
	long int	length;
	short int	version;
	short int	status;
} tsh_t;

typedef struct timeServerReply {
	tsh_t		header;
	time_t		time;
} tstr_t;

typedef enum
{
	SS_UNKNOWN = 0,
	SS_CONNECTING,
	SS_SENDING,
	SS_LISTENING,
	SS_TERMINATING,
	SS_DONE
} SessionState;
/****************************************************************************/


/**** Local Variables ****/

SPXHeader
		connectPacket;

struct
{
	SPXHeader
			header;
	tsh_t	timeRequest;
} sendPacket;

struct
{
	SPXHeader
			header;
	tstr_t	timeReply;
} listen;

ECB		connectECB,
		sendECB,
		listenECB[LISTEN_ECB_COUNT];

int	socketNumber;
/****************************************************************************/


/**** Main - SPX example client
 *
 * Input:	SPX server address		nnnnnnnn:aaaaaaaaaaaa;ssss
 *
 * Output:	None
 *
 * Display:	Yes
 *
 * Returns:	Exit Code				0 = Successful
 *									1 = Initialization Error
 *									2 = Protocol Error
 *									3 = Programming Error
 *
 * Comment:	Usage:  SPXTCLI
 *
 ****/

void main(int argc, char *argv[])
{
	BYTE		majorRev,
				minorRev;
	WORD		maxConn,
				availConn;
	int			connectionID,
				i;
	IPXAddress	destination;
	SessionState
				sessionState = SS_UNKNOWN;

/* Check for proper command line arguments */
	if (argc != 2)
	{
		printf("Usage: SPXTCLI net:node;socket\n");
		exit(1);
	}
/* Establish that SPX is loaded and initialize interface */
	if (!SPXInitialize(&majorRev, &minorRev, &maxConn, &availConn))
	{
		printf("SPX not installed; terminating.\n");
		exit(1);
	}

/* Get the time server's address from the command line */
	if (!SPXParseAddress(argv[1], &destination))
	{
		printf("Invalid SPX address; terminating.\n");
		exit(1);
	}

/* Get a dynamic reply socket */
	socketNumber = 0;
	if (IPXOpenSocket((BYTE *) &socketNumber, 0) != 0)
	{
		printf("Unable to open reply socket; terminating.\n");
		exit(1);
	}

/* Initialize the listen ECB & packet */
	for (i = 0; i < LISTEN_ECB_COUNT; i++)
	{
		listenECB[i].socketNumber = socketNumber;
		listenECB[i].fragmentCount = 1;
		listenECB[i].fragmentDescriptor[0].address =
			(char far *) &listen;
		listenECB[i].fragmentDescriptor[0].size = sizeof(listen);
		SPXListenForSequencedPacket(&listenECB[i]);
	}

/* Initialize the send ECB & packet */
	sendECB.fragmentCount = 1;
	sendECB.fragmentDescriptor[0].address =
		(void far *) &sendPacket;
	sendPacket.header.connectionControl = 0x010;
	sendECB.fragmentDescriptor[0].size = sizeof(sendPacket);
	sendPacket.timeRequest.function = TS_TIME_REQUEST;
	sendPacket.timeRequest.length = sizeof(tsh_t);
	sendPacket.timeRequest.version = TS_TR_VER;
	sendPacket.timeRequest.status = 0;

/* Initialize the connect packet */
	connectPacket.destination = destination;
	connectECB.socketNumber = socketNumber;
	connectECB.fragmentCount = 1;
	connectECB.fragmentDescriptor[0].address =
		(void far *) &connectPacket;
	connectECB.fragmentDescriptor[0].size = sizeof(SPXHeader);

/* Begin the connection establish process */
	if (SPXEstablishConnection(0, 1, (WORD *) &connectionID,
			&connectECB) != 0)
	{
		printf("Unable to establish connection; terminating.\n");
		exit(1);
	}
	else
		printf("Session 0x%04X starting\n", connectionID);
	sessionState = SS_CONNECTING;

/* Loop until request complete */
	while (sessionState != SS_DONE)
	{
		IPXRelinquishControl();

		if (kbhit())
		{
			getch();
			break;
		}

		switch (sessionState)
		{
		case SS_CONNECTING:
		/* Wait for connect request to complete */
			if (connectECB.inUseFlag == 0)
			{
				if (connectECB.completionCode)
				{
					printf("Session 0x%04X: Connect failed! 0x%04X\n",
							connectionID, connectECB.completionCode);
					exit(2);
				}
				SPXSendSequencedPacket(connectPacket.sourceConnectionID,
						&sendECB);

				sessionState = SS_SENDING;
			}
			break;

		case SS_SENDING:
		/* Wait for time request packet send to complete */
			if (sendECB.inUseFlag == 0)
			{
				if (sendECB.completionCode)
					exit(printf("Session 0x%04X: Send failed! 0x%04X\n",
							connectionID, sendECB.completionCode));

			/* Listen is already posted */
			 	sessionState = SS_LISTENING;
			}
			break;

		case SS_LISTENING:
		/* Wait for an answer to come back */
			for (i = 0; i < LISTEN_ECB_COUNT; i++)
			{
				if (listenECB[i].inUseFlag == 0)
				{
					if (listenECB[i].completionCode)
					{
						printf("Receive error 0x%04X\n",
								listenECB[i].completionCode);
						exit(2);
					}

					if (listen.timeReply.header.status ==
							TS_REQUEST_COMPLETE)
						printf("Time server returned %s\n",
							asctime(localtime(&listen.timeReply.time)));
					else
						printf("Time server returned ERROR %d\n",
								listen.timeReply.header.status);

					SPXTerminateConnection(connectionID, &connectECB);

					sessionState = SS_TERMINATING;
				}
			}
			break;

		case SS_TERMINATING:
		/* Wait for termination to complete */
			if (connectECB.inUseFlag == 0)
			{
				if (connectECB.completionCode)
					exit(printf("Session 0x%04X: Terminate failed! 0x%04X\n",
							connectionID, connectECB.completionCode));

				sessionState = SS_DONE;
			}
			break;
		}
	}
/* Clean up the IPX socket */
	IPXCloseSocket(socketNumber);
}
/****************************************************************************/


/**** SPX Address Parsing routines
 *
 * Input:	char		*addressBuffer;
 *
 * Output:	IPX_ADDR	*parsedAddress;
 *
 *
 * Display:	None
 *
 * Returns:	int			 result;		TRUE if addressBuffer was valid
 *
 * Comment:	Address must be in nnnnnnnn:aaaaaaaaaaaa;ssss notation with
 *			leading zeros optional.  nnnnnnnn = Network number,
 *			aaaaaaaaaaaa = Node number, ssss = Socket number.
 ****/

static int
InsertAddress(char *string, char *buf, int hexBytes)
{
	int			 i,
				 j,
				 value;
	char		 c;

	for (i = 0; i < hexBytes; i++)
	{
	/* build a byte from two nibbles */
		value = 0;
		for (j = 0; j < 2; j++)
		{
			value <<= 4;
			if ((c = toupper(*string)) >= '0' && c <= '9')
				value += c - '0';
			else if (c >= 'A' && c <= 'F')
				value += c - 'A' + 10;
			else
				return FALSE;
			string++;
		}
		*buf++ = value;
	}
	return TRUE;
}
/****************************************************************************/

static int
SPXParseAddress(char *text, struct IPXAddress *spxAddr)
{
	char		 temp[13],
				*cp;
	int			 len;

/* Does the number contain a net number? */
	if ((cp = strchr(text, ':')) == NULL)
		return FALSE;

/* Parse out the net number */
	*cp++ = '\0';
	if ((len = strlen(text)) < 1 || len > 8)
		return FALSE;
	strcpy(temp, "00000000");
	strcpy(&temp[8 - strlen(text)], text);
	if (!InsertAddress(temp, spxAddr->network, 4))
		return FALSE;
/* Skip over the Network Number portion */
	text = cp;

/* Does the number contain a Socket number? */
	if ((cp = strchr(text, ';')) == NULL)
		return FALSE;

/* Parse out the socket number */
	*cp++ = '\0';
	if ((len = strlen(text)) < 1 || len > 4)
		return FALSE;
	strcpy(temp, "0000");
	strcpy(&temp[4 - strlen(cp)], cp);
	if (!InsertAddress(temp, (char *)&spxAddr->socket, 2))
		return FALSE;

/* Parse out the node number */
	if ((len = strlen(text)) < 1 || len > 12)
		return FALSE;
	strcpy(temp, "000000000000");
	strcpy(&temp[12 - strlen(text)], text);
	if (!InsertAddress(temp, spxAddr->node, 6))
		return FALSE;
	
	return TRUE;
}
/****************************************************************************/
/****************************************************************************/


