
#ifndef _TIUSER_
#define _TIUSER_

#define	O_NONBLOCK	O_NDELAY

#ifndef EAGAIN
#define EAGAIN  -1
#endif



/* Error values */
#define	TACCES			1
#define	TBADADDR		2
#define	TBADDATA		3
#define	TBADF			4
#define	TBADFLAG		5
#define	TBADOPT			6
#define	TBADSEQ			7
#define	TBUFOVFLW		8
#define	TFLOW			9
#define	TLOOK			10
#define	TNOADDR			11
#define	TNODATA			12
#define	TNOREL			13
#define	TNOTSUPPORT		14
#define	TOUTSTATE		15
#define	TSTATECHNG		16
#define TSYSERR			17
#define	TNOUDERR		18
#define	TNODIS			19
#define	TNOSTRUCTYPE	20
#define TBADNAME		21
#define	TBADQLEN		22
#define	TADDRBUSY		23

/* t_look events */
#define	T_LISTEN		1
#define	T_CONNECT		2
#define	T_DATA			3
#define	T_EXDATA		4
#define	T_DISCONNECT	5
#define	T_ORDREL		6
#define	T_ERROR			7
#define	T_UDERR			8
#define	T_GODATA		9
#define	T_GOEXDATA		10
#define	T_EVENTS		11

/* Flag definitions */
#define	T_EXPEDITED		0x01
#define	T_MORE			0x02
#define	T_NEGOTIATE		0x04
#define	T_CHECK			0x08
#define	T_DEFAULT		0x10
#define T_SUCCESS		0x20
#define	T_FAILURE		0x40

#define t_errno		*__get_t_errno_ptr()
extern int t_errno;

struct t_info {
	long	addr;
	long	options;
	long	tsdu;
	long	etsdu;
	long	connect;
	long	discon;
	long	servtype;
};

/* Service types */
#define T_COTS		1	/* Connection-mode service */
#define	T_COTS_ORD	2	/* Connection service with orderly release */
#define	T_CLTS		3	/* Connectionless-mode service */

struct netbuf {
	unsigned	maxlen;
	unsigned	len;
	char		* buf;
};

struct t_bind {
	struct netbuf	addr;
	unsigned	qlen;
};

struct t_optmgmt {
	struct netbuf	opt;
	long	flags;
};

struct t_discon {
	struct netbuf	udata;
	int		reason;
	int		sequence;
};

struct t_call {
	struct netbuf	addr;
	struct netbuf	opt;
	struct netbuf	udata;
	int		sequence;
};

struct t_unitdata {
	struct netbuf	addr;
	struct netbuf	opt;
	struct netbuf	udata;
};

struct t_uderr {
	struct netbuf	addr;
	struct netbuf	opt;
	long		error;
};

/* t_alloc structure types */
#define	T_BIND		0x1
#define	T_CALL			0x2
#define T_OPTMGMT		0x4
#define	T_DIS			0x8
#define	T_UNITDATA		0x10
#define	T_UDERROR		0x20
#define	T_INFO			0x40

/* XTI names for t_alloc structure types */
#define	T_BIND_STR		T_BIND
#define T_OPTMGMT_STR	T_OPTMGMT
#define	T_CALL_STR		T_CALL
#define	T_DIS_STR		T_DIS
#define	T_UNITDATA_STR	T_UNITDATA
#define	T_UDERROR_STR	T_UDERROR
#define	T_INFO_STR		T_INFO

/* t_alloc field identifiers */
#define	T_ADDR			0x1000
#define	T_OPT			0x2000
#define	T_UDATA			0x4000
#define	T_ALL			0x8000

/* State values */
#define	T_UNINIT	0	/* added to match xti state tables */
#define	T_UNBND		1	/* unbound */
#define	T_IDLE		2	/* idle */
#define	T_OUTCON	3	/* outgoing connection pending */
#define	T_INCON		4	/* incoming connection pending */
#define	T_DATAXFER	5	/* data transfer */
#define	T_OUTREL	6	/* outgoing orderly release */
#define	T_INREL		7	/* incoming orderly release */

/* general purpose defines */
#define	T_YES		1
#define	T_NO		0
#define	T_UNUSED	-1
#define	T_NULL		0
#define	T_ABSREQ	0x8000

/* t_event flag values */
#define	T_SAME		0x0
#define	T_ENABLE	0x1
#define T_DISABLE	0x2


extern	char	* t_errlist[];
extern	int	t_nerr;

/* ------------------------OSI specific Options-------------------------*/

#define	T_CLASS0	0
#define	T_CLASS1	1
#define	T_CLASS2	2
#define	T_CLASS3	3
#define	T_CLASS4	4

/* priorities */
#define	T_PRITOP	0
#define	T_PRIHIGH	1
#define	T_PRIMID	2
#define	T_PRILOW	3
#define	T_PRIDFLT	4

/* protection levels */
#define	T_NOPROTECT			1
#define	T_PASSIVEPROTECT	2
#define	T_ACTIVEPROTECT		4

/* default value for the length of TPDU's */
#define	T_LTPDUDFLT	128

/* rate structure */
struct rate {
	long	targetvalue;
	long	minacceptvalue;
};

/* reqvalue structure */
struct reqvalue {
	struct rate	called;
	struct rate	calling;
};

/* throughput structure */
struct thrpt {
	struct reqvalue	maxthrpt;
	struct reqvalue	avgthrpt;
};

/* management structure */
struct management {
	short	dflt;		/* T_YES to use default values or T_NO to use values in structure */
	int	ltpdu;		/* maximum length of TPDU */
	short	reastime;	/* reassignment time (in seconds) */
	char	class;		/* preferred class */
	char	altclass;	/* alternative class */
	char	extform;	/* extended format: T_YES or T_NO */
	char	flowctrl;	/* flow control: T_YES or T_NO */
	char	checksum;
	char	netexp;		/* network expedited data */
	char	netrecptcf;	/* receipt confirmation */
};

/* ISO connection-oriented options */
struct isoco_options {
	struct thrpt	throughput;
	struct reqvalue	transdel;	/* transit delay */
	struct rate	reserrorrate;	/* residual error rate */
	struct rate	transffailprob;	/* transfer failure problem */
	struct rate	estfailprob;	/* connection establishment failure problem */
	struct rate	relfailprob;	/* connection release failure problem */
	struct rate	estdelay;	/* connection establishment delay */
	struct rate	reldelay;	/* connection release delay */
	struct netbuf	connresil;	/* connection resilience */
	unsigned short	protection;
	short		priority;
	struct management mngmt;	/* management parameters */
	char		expd;		/* expedited data: T_YES or T_NO */
};

/* ISO connectionless options */
struct isocl_options {
	struct rate	transdel;	/* transit delay */
	struct rate	reserrorrate;	/* residual error rate */
	unsigned short	protection;
	short		priority;
};


/*---------------------TCP specific Options--------------------------*/
	

/* TCP Precedence Levels */
#define	T_ROUTINE	0
#define	T_PRIORITY	1
#define	T_IMMEDIATE	2
#define	T_FLASH		3
#define	T_OVERRIDEFLASH	4
#define	T_CRITIC_ECP	5
#define	T_INETCONTROL	6
#define	T_NETCONTROL	7

/* TCP security options structure */
struct secoptions {
	short	security;
	short	compartment;
	short	handling;	/* handling restrictions */
	long	tcc;		/* transmission control code */
};

/* TCP options */
struct tcp_options {
	short		precedence;
	long		timeout;	/* abort timeout */
	long		max_seg_size;
	struct secoptions secopt;	/* security options */
};


/*----------------------SPX\IPX specific Options-----------------------*/

#ifndef _TISPXIPX_
#define _TISPXIPX_

typedef struct ipxaddr_s {
	unsigned char	ipxa_net[4];
	unsigned char	ipxa_node[6];
	unsigned char	ipxa_socket[2];
} IPX_ADDR;

typedef	struct ipxopt_s {
	unsigned char	ipx_type;		/* type field for ipx header */
	unsigned char	ipx_pad1[3];	/* pad structure to 4 bytes, set to 0's */
	unsigned char	ipx_hops;		/* Transport Control (hop count) */
	unsigned char	ipx_pad2[3];	/* pad structure to 8 bytes, set to 0's */
} IPX_OPTS;

typedef struct spxopt_s {
	unsigned char	spx_connectionID[2];
	unsigned char	spx_allocationNumber[2];
	unsigned char	spx_pad1[4];	/* pad structure to 8 bytes, set to 0's */
} SPX_OPTS;

typedef struct spx_optmgmt {
	unsigned char	spxo_retry_count;
	unsigned char	spxo_watchdog_flag;
	unsigned long	spxo_min_retry_delay;
	unsigned char	spxo_pad2[2];	/* pad structure to 8 bytes, set to 0's */
} SPX_OPTMGMT;

/* new version 4.00 spxs option structure definition */
/* New structure is the same for all three calls t_listen, t_connect, t_optmgmt. */
  
#define			OPTIONS_VERSION		1UL

typedef struct spx_options	{
	long				versionNumber;
	SPX_OPTS			spxOpts;
	SPX_OPTMGMT			spxOptMgmt;
	unsigned long  		spxIIPacketSizeNegotiate;
	unsigned long		spxIIInboundPacketSize;	   	//this is return option only (cannot be negotiated) - it return size of max inbound packet size
	unsigned long		spxIIOutboundPacketSize;	//this is return option only (cannot be negotiated) - it return size of max outbound packet size
} SPX_OPTIONS;

/* Misc. byte aligned data access macros */
#ifndef		BE16_TO_U16
#define	BE16_TO_U16(a)	((((u16)((u8 *)a)[0] << (u16)8) | ((u16)((u8 *)a)[1] & 0xFF)) & (u16)0xFFFF)
#define	BE32_TO_U32(a)	((((u32)((u8 *)a)[0] & 0xFF) << (u32)24) | (((u32)((u8 *)a)[1] & 0xFF) << (u32)16) | (((u32)((u8 *)a)[2] & 0xFF) << (u32)8)  | ((u32)((u8 *)a)[3] & 0xFF))
#define	U16_TO_BE16(u,a) ((((u8 *)a)[0] = (u8)((u) >> 8)), (((u8 *)a)[1] = (u8)(u)))
#define	U32_TO_BE32(u,a) ((((u8 *)a)[0] = (u8)((u) >> 24)), (((u8 *)a)[1] = (u8)((u) >> 16)), (((u8 *)a)[2] = (u8)((u) >> 8)),(((u8 *)a)[3] = (u8)(u)))
#endif

/* T_DISCONNECT reason codes for SPX */
#define TLI_SPX_CONNECTION_FAILED		0xed
#define TLI_SPX_CONNECTION_TERMINATED	0xec	/* T_DISCONNECT from remote */
#define TLI_SPX_MALFORMED_PACKET		0xfe
#define TLI_SPX_PACKET_OVERFLOW			0xfd	/* This is the preferred one */
#define	TLI_SPX_UNREACHABLE_DEST		0x70	/* route not found */

/* t_rcvuderr uderr.error codes for IPX */
#define TLI_IPX_MALFORMED_ADDRESS		0xfe
#define TLI_IPX_PACKET_OVERFLOW			0xfd

/* SPX Option Management values */
#define SPX_RETRY_DEFAULT		10
#define SPX_RETRY_MIN			3
#define SPX_RETRY_MAX			50
#define SPX_WATCHDOG_ON			1
#define SPX_WATCHDOG_OFF		0
#define SPX_WATCHDOG_DEFAULT	1
#define SPX_SIZE_NEGOTIATE_ON	0
#define SPX_SIZE_NEGOTIATE_OFF	!SPX_SIZE_NEGOTIATE_ON
#endif


/*---------------------end of Options---------------------------------*/




#ifndef	NO_PROTOTYPES

extern	int	 t_accept (int fd, int resfd, struct t_call * call);
extern	char *t_alloc (int fd, int struct_type, int fields);
extern	int	 t_bind (int fd, struct t_bind * req, struct t_bind  * ret);
extern	int	 t_blocking(int fd);
extern	int	 t_close (int fd);
extern	int	 t_connect (int fd, struct t_call  * sndcall, struct t_call  * rcvcall);
extern	void t_error (char  * errmsg);
extern	int	 t_event(int fd, int (*func)(int fd, int event, void *parm), 
					int flag, void *parm);
extern	int	 t_free (char  * ptr, int struct_type);
extern	int	 t_getinfo (int fd, struct t_info  * info);
extern	int	 t_getstate (int fd);
extern	int	 t_listen (int fd, struct t_call  * call);
extern	int	 t_look (int fd);
extern	int	 t_nonblocking(int fd);
extern	int	 t_open (char  * path, int	oflag, struct t_info  * info);
extern	int	 t_optmgmt (int fd, struct t_optmgmt  * req, struct t_optmgmt  * ret);
extern	int	 t_rcv (int fd, char  * buf, unsigned nbytes, int  * flags);
extern	int	 t_rcvconnect (int fd, struct t_call  * call);
extern	int	 t_rcvdis (int fd, struct t_discon  * discon);
extern	int	 t_rcvrel (int fd);
extern	int	 t_rcvudata (int fd, struct t_unitdata  * unitdata, int  * flags);
extern	int	 t_rcvuderr (int fd, struct t_uderr  * uderr);
extern	int	 t_snd (int fd, char  * buf, unsigned nbytes, int flags);
extern	int	 t_snddis (int fd, struct t_call  * call);
extern	int	 t_sndrel (int fd);
extern	int	 t_sndudata (int fd, struct t_unitdata  * unitdata);
extern	int	 t_sync(int fd);
extern	int	 t_unbind(int fd);

#endif	/* ifndef no_prototypes */


#endif	/* ifdef _TIUSER */
