
/**/
/*                                                                       */
/*            TLI Based Time Server using ConnectionLESS IPX             */
/*                                                                       */
/*   This routine will act as a time server.  It will use a well         */
/*   know socket to service requests.  It will accept IPX datagram       */
/*   time server requests on this socket.  It will reply to these        */
/*   requests with the current time returned in the IPX data portion.    */
/*   It will then resume listening for more requests.                    */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <time.h>
#include <fcntl.h>
/*   Now incorporated in tiuser.h #include <tispxipx.h> */
#include <tiuser.h>
#include <sap.h>
#include "mrdnxt.h"   /* Missing nxt.h declarations       */
#include "timeserv.h" /* Timeserver specific declarations */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);

main()
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* TZ environment variable                    */
	IPXAddress  myaddress;    /* My network,node,socket                     */
	IPXAddress  clientaddress;  /* Clients network,node,socket              */
	WORD        serverSocket;   /* Temp holding area for server socke       */
	int         received_data;  /* Boolean : did I receive data yet ?       */
	int         i;            /* scratch index                              */

/*   TLI structures and variables                                          */

	int               fd;              /* identifies our communication end  */
	struct t_bind     tbind;           /* to bind to a socket               */

	struct t_unitdata send_tunitdata;  /* sending datagram buffers, structs */
	IPX_OPTS          send_opts;       /* protocol options                  */
	int               send_flags;     
	unsigned char     ipx_data[512];   /* used for both send and receive    */

	struct t_unitdata recv_tunitdata;   /* receive datagram buffers, structs*/
	int               recv_flags;     

/*  */

/*   Get and display the time   */

	time_zone = getenv("TZ");
	if (time_zone == NULL)
		{
		printf("\nTime zone variable (TZ=) must be set before running server\n");
		printf("Time server aborted\n\n");
		return;
		}
	else
		printf("\nTime zone is %s\n",time_zone);

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is %s\n", asctime(time_block) );

/*
 *   Open a connection endpoint using the ipx protocol device.
 *   A unique id for this connection endpoint will be returned in fd.
 *   We open the endpoint as read and write, no info to be returned.
 *   We open no delay so we can abort from the keyboard.
 */
	if (( fd = t_open("/dev/nipx", O_RDWR|O_NDELAY, NULL )) == -1)
		{
		t_error("Open of IPX device failed, program aborted");
		return;
		}

/*
 *   Bind to a well known for socket chosen for the timeserver to use.
 *   Our network and node need not be set, they are known to the provider
 *   and will be returned to us if bound successfully.  We use the
 *   same structure to pass and receive info from the provider.
 */
	serverSocket=TIME_SERVER_SOCKET;
	memcpy(myaddress.socket,&serverSocket,2);
	tbind.addr.len    = sizeof(IPXAddress);
	tbind.addr.maxlen = sizeof(IPXAddress);
	tbind.addr.buf    = (char *) &myaddress;
	tbind.qlen = 0;	/* qlen is meaningless for connectionless transports */

	if (t_bind(fd, &tbind, &tbind) == -1)
		{
		t_error("Bind to server socket failed, program aborted");
		return;
		}

/*
 *   Display the local network and node returned from the t_bind and
 *   the well known server socket.
 */
	printf("TLI-IPX based timeserver started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*
 *    This is the main server loop.
 *    Since we opened in non-blocking mode we will look for data/sleep
 *    until some actually arrives. If a key is pressed, we abort.
 */
	while (!kbhit())
		{
		recv_tunitdata.addr.buf     = (char *)&clientaddress;
		recv_tunitdata.addr.maxlen  = sizeof(IPXAddress);
		recv_tunitdata.udata.buf    = (char *)ipx_data;
		recv_tunitdata.udata.maxlen = 512;
		recv_tunitdata.opt.maxlen   = 0;	  /* dont care to see them */

	/*
	 *   Loop, When something comes in t_rcvudata will return 0,
	 *   elsewise it should return -1 and t_errno will be set
	 *   to TNODATA ( no data available yet )
	 */
		received_data = FALSE;		 
		while ( ! received_data )
			{
			sleep(1); 
			if (kbhit())    
				{
				printf("Program canceled from keyboard\n");
				return;
				}
			if ( t_rcvudata(fd, &recv_tunitdata, &recv_flags ) == 0)
				received_data = TRUE;
			else
				{
				if (t_errno != TNODATA)
					{
					t_error("Fatal error during t_recvudata");
					return;
					}
				}
			}  /* end while ! received_data */


	/*   Check to see if this is really a time request   */

		printf("Packet received on time server socket from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",clientaddress.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",clientaddress.node[i]);
		printf("\n\n");

		if (ipx_data[0] != TIME_REQUEST)
			printf("\nNon TIME REQUEST received on socket, ignored\n\n");
		else
			{
		/*
		 *   This seems to be a valid request, build a time packet and
		 *   send it back to the client.
		 */
			printf("\nValid TIME REQUEST received, processing now.\n");

		/*   Setup the packet type in the options structure */

			send_opts.ipx_type        = IPX_PACKET_TYPE; 
			send_tunitdata.opt.buf    = (char *)&send_opts;
			send_tunitdata.opt.len    = sizeof(IPX_OPTS);
			send_tunitdata.opt.maxlen = sizeof(IPX_OPTS);

		/*   Setup the address to send the reply to */

			send_tunitdata.addr.buf    = (char *)&clientaddress;
			send_tunitdata.addr.len    = sizeof(IPXAddress);
			send_tunitdata.addr.maxlen = sizeof(IPXAddress);

		/*   Setup the data buffer pointer and max length */

			send_tunitdata.udata.buf    = (char *)ipx_data;
			send_tunitdata.udata.maxlen = 512;

		/*
		 *   Get the time into the data area at the last possible moment
		 *   and then send the time packet on its merry way.
		 */
			ipx_data[0] = TIME_REPLY;              /* time reply operation   */
			time_of_day = time(NULL);              /* get latest time        */
			time_zone = getenv("TZ");				   /* and time zone          */
			memcpy(&ipx_data[1], &time_of_day, sizeof(time_t));
			strcpy(&ipx_data[1+sizeof(time_t)],time_zone);

		/*   Stuff the real length in, and let it fly */

			send_tunitdata.udata.len = 1 	                 /* time reply       */
												+ sizeof(time_t)	  /* time             */
												+ strlen(time_zone) /* time zone string */
												+ 1;                /* string null      */

			if (t_sndudata(fd, &send_tunitdata) == -1)
				{
				t_error("Fatal error, time reply send unsuccessful");
				return;
				}
				
			printf("TIME REPLY sent successfully.\n\nPress any key to abort program\n\n");

			} /* valid TIME REQUEST */
			
		} /* end while !kbhit() */

	printf("\nProgram canceled from keyboard\n\n");

return;			
}  /* end main() */

