
/**/
/*                                                                       */
/*	    TLI Based Time Query Client using SPX protocol provider           */
/*                                                                       */
/*   This routine will query the user for network and node address of    */
/*   any SPX time server. It will attempt a connection with that server. */
/*   If successful, it It will then send a time request to contained     */
/*   with SPX data.  When the response is received, the routine will     */
/*   set the local and then attempt to disconnect gracefully.            */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"    /* Missing defines in nxt.h */
#include "timeserv.h"  /* Time server definitions  */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main() 
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	time_t     *time_p;       /* used for conversion                        */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* ptr to TZ= string                          */

	WORD        serverSocket;   /* Temp WORD area for server socket         */
	int         connection_open; /* Boolean, connection currently open ?    */
	int         data_received;   /* Boolean, data packet received ?         */

	IPXAddress  myaddress;    /* My network,node,socket                     */
	IPXAddress  server;       /* Server network,node,socket                 */

	int         i;            /* scratch index                              */

/*   TLI structures and variables                                          */

	int               fd;              /* identifies our communication end  */
	struct t_bind     tbind;           /* to bind to a socket               */

	struct t_call     sndcall;         /* info needed to establish connectn */
	int               send_flags;
	int               send_size;       /* size of data sent                 */
	unsigned char     spx_data[512];   /* used for both send and receive    */

	int					recv_flags;     


/*  */

/*   Get and display the old time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("\nThe time was %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (*time_zone == NULL)
		printf("Time zone was NULL\n");
	else
		printf("Time zone was %s\n\n",time_zone);

/*
 *   Ask the user which timeserver he wants to use. We do this because
 *   advertising/geting server only works from a workstation perspective
 *   if there is an intermediate router or if the workstation implements
 *   a complete SAP handler, which is not the case in this simple example.
 */
	printf("Please enter the timeserver address\n\n");
	GetIPXAddress(&server);
	serverSocket=TIME_SERVER_SOCKET;			 /* server socket is well known */
	memcpy(server.socket,&serverSocket,2);

/*
 *   Open a connection endpoint using the SPX protocol device.
 *   A unique id for this connection endpoint will be returned in fd.
 *   We open the endpoint as read and write, no info to be returned.
 *   We open no delay so we can abort from the keyboard.
 */
	if (( fd = t_open("/dev/nspx", O_RDWR|O_NDELAY, NULL )) == -1)
		{
		t_error("Open of SPX device failed, program aborted");
		return;
		}

/*
 *   Bind to a dynamic for socket chosen for this client to use.
 *   Our network and node need not be set, they are known to the provider
 *   and will be returned to us if bound successfully.  We use the
 *   same structure to pass and receive info from the provider.
 */
	memset(myaddress.socket,0,2);
	tbind.addr.len    = sizeof(IPXAddress);
	tbind.addr.maxlen = sizeof(IPXAddress);
	tbind.addr.buf    = (char *) &myaddress;
	tbind.qlen = 0;	/* we accept no connections, we are a client only */

	if (t_bind(fd, &tbind, &tbind) == -1)
		{
		t_error("Bind to dynamic socket failed, program aborted");
		return;
		}

/*
 *   Display the local network and node returned from the t_bind and
 *   the well known server socket.
 */
	printf("TLI-SPX based time query client started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*
 *   Open a connection to the time server in two steps. First send
 *   the t_connect to initiate a connection.  Since we opened in
 *   non blocking mode we must then wait until the connection
 *   is completed (t_rcvconnect returns successfull). This way we
 *   can abort from the keyboard if necessary.
 */

/*   Setup area describing connection information */

	sndcall.addr.buf     = (char *)&server;
	sndcall.addr.maxlen  = sizeof(IPXAddress);
	sndcall.addr.len     = sizeof(IPXAddress);
	sndcall.udata.len    = 0;  /* dont want to send any connect user data */
	sndcall.opt.len      = 0;  /* dont have to pass any options */

	if (t_connect( fd, &sndcall, NULL) == -1)
		if (t_errno != TNODATA)	/* this was not just a O_NDELAY indication */
			{
			t_error("T_connect fatal error, program aborted");
			return;
			}
/*
 *   Loop, When the connection is completed t_rcvconnect will return 0,
 *   elsewise it should return -1 and t_errno will be set
 *   to TNODATA ( no data available yet ) 
 */
	connection_open = FALSE;
	while ( ! connection_open )
		{
		sleep(1); 
		if (kbhit())    
			{
			printf("Program canceled from keyboard\n");
			return;
			}
		if ( t_rcvconnect(fd, &sndcall ) == 0)	/* we received the confirm */
			connection_open = TRUE;
		else
			{
			if (t_errno != TNODATA)
				{
				t_error("Fatal error during t_rcvconnect");
				return;
				}
			}
		}  /* end while ! connection_open */

	printf("Established connection with server");

/*   Send a TIME REQUEST to the time server. */

	spx_data[0] = TIME_REQUEST;   /* time request operation   */
	send_size = 1;
	send_flags = 0;

	if (t_snd(fd, spx_data, send_size, send_flags) == -1)
		{
		t_error("Fatal error, time request send unsuccessful");
		return;
		}
	else
		printf("Time REQUEST sent, waiting for REPLY\n\n");

/*
 *   Listen on my socket for the reply.  Since we opened as
 *   non blocking, t_rcv may fail for a while returning
 *   TNODATA until data is received.  We may also encounter
 *   a disconnect via a TLOOK error so handle that here also.
 */
	while ( connection_open ) /* service all data packets received */
		{
		data_received = FALSE;
		while ( ! data_received )  /* wait for some data to come */
			{
			sleep(1); 
			if (kbhit())    
				{
				printf("Program canceled from keyboard\n");
				return;
				}

			if ( t_rcv(fd, spx_data, 512, &recv_flags) != -1 )
				data_received = TRUE;
			else
				{ /* This failure may be 1. T_LOOK, A. Disconnect received
			 		*							            B. Other Fatal error
					*	                   2. TNODATA - No data yet, thats OK
			 		*							 3. Fatal Error
			 		*/
				if (t_errno == TLOOK)
					{
					if(t_look(fd) == T_DISCONNECT)
						{ /*  Process the disconnect */
						if (t_rcvdis(fd,NULL) == -1)
							{
							t_error("T_rcvdis fails, program aborted");
							return;
							}
						connection_open = FALSE;
						data_received   = TRUE; /* set to exit while gracefully */
						}
					else
						{
						t_error("T_look error during t_rcv, program aborted");
						return;
						}
					}
				else /* not a T_LOOK, better be TNODATA, no data error */
					if (t_errno != TNODATA)
						{
						t_error("Fatal error during t_rcv");
						return;
						}
				} /* end checking t_rcv failure */

			}  /* end while ! data_received */

 	/*
 	 *   After that last while we either have real data to process
 	 *   or the connection was closed out.  If there is still
 	 *   a connection open, it means there is data, process it.
 	 */
 		if (connection_open)
 			{
			if (spx_data[0] != TIME_REPLY)
				{
				printf("\nNon TIME REPLY packet received on socket, ignored\n\n");
				break;
				}
			else
			/*   This seems to be a valid reply, set the system time
			 *   and timezone from the values received in the packet.
			 */
				{
				printf("\nValid TIME REPLY received, processing now.\n");
				time_zone=malloc(strlen(&spx_data[1+sizeof(time_t)]) + 8 );
				strcpy(time_zone,"TZ=");
				strcat(time_zone,&spx_data[1+sizeof(time_t)]);
				putenv(time_zone);     /* setting the TZ environment var */
				tzset();               /* and making it permanent        */
				free(time_zone);
				time_p      = (time_t *)&spx_data[1]; /* point to time  */
				time_of_day = *time_p;                /* get latest time */
				stime(&time_of_day);                  /* and set it      */

				} /* end valid TIME REPLY */
		/*
 		 *   Now that we have received and processed a valid
 		 *   time reply, we can close down the connection gracefully.
 		 */
			if (t_snddis(fd, NULL) == -1 )
				{
				t_error("Fatal error on t_snddis, program aborted");
				return;
				}
 			connection_open = FALSE;

 			} /* end if connection_open */

	 } /* end while connection_open */

/*
 *   Finally, Get and display the new time
 *   so that we can tell the whole thing ran correctly
 */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is now %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (time_zone == NULL)
		printf("Time zone is now NULL\n\n");
	else
		printf("Time zone is now %s\n\n",time_zone);

return;			
}  /* end main() */

