
/**/
/*                                                                       */
/*        TLI Based Time Server using SPX protocol provider              */
/*	    	                                                                 */
/*   This routine will act as a time server.  Connection                 */
/*   establishment and termination with a client will be handled         */
/*   on its well known socket. It will accept time requests only         */
/*   on an established connection. It will reply to that request         */
/*   with the current time returned in the SPX data portion.             */
/*   All other connection attempts will be rejected until the            */
/*   current connection is terminated.                                   */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"    /* Missing defines in nxt.h */
#include "timeserv.h"  /* Time server definitions  */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main()
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* TZ environment variable                    */

	IPXAddress  myaddress;    /* My network,node,socket                     */
	IPXAddress  clientaddress;/* Client connected to me                     */
	WORD        serverSocket; /* Temp holding area for server socket        */
	int         i;            /* scratch index                              */

	int         connection_received; /* Boolean, connection received ?      */
	int         connection_open;     /* Boolean, connection currently open ?*/
	int         data_received;       /* Boolean, data packet received ?     */

/*   TLI structures and variables                                          */

	int               fd;              /* identifies our communication end  */
	struct t_bind     tbind;           /* to bind to a socket               */
	struct t_call     tcall;           /* to accept connections             */

	int               send_flags;      /* flags for t_snd                   */
	int               send_size;       /* size of data to send              */
	unsigned char     spx_data[512];   /* used for both send and receive    */

	int               recv_flags;      /* flags returned from t_rcv         */

/*  */

/*   Get and display the time   */

	time_zone = getenv("TZ");
	if (time_zone == NULL)
		{
		printf("\nTime zone variable (TZ=) must be set before running server\n");
		printf("Time server aborted\n\n");
		return;
		}
	else
		printf("\nTime zone is %s\n",time_zone);

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is %s\n", asctime(time_block) );


/*
 *   Open a connection endpoint using the SPX protocol device.
 *   A unique id for this connection endpoint will be returned in fd.
 *   We open the endpoint as read and write, no info to be returned.
 *   We open no delay so we can abort from the keyboard.
 */
	if (( fd = t_open("/dev/nspx", O_RDWR|O_NDELAY, NULL )) == -1)
		{
		t_error("Open of SPX device failed, program aborted");
		return;
		}

/*
 *   Bind to a well known for socket chosen for the timeserver to use.
 *   Our network and node need not be set, they are known to the provider
 *   and will be returned to us if bound successfully.  We use the
 *   same structure to pass and receive info from the provider.
 */
	serverSocket=TIME_SERVER_SOCKET;
	memcpy(myaddress.socket,&serverSocket,2);
	tbind.addr.len    = sizeof(IPXAddress);
	tbind.addr.maxlen = sizeof(IPXAddress);
	tbind.addr.buf    = (char *) &myaddress;
	tbind.qlen = 1;	/* as a simple server I accept only one connection */

	if (t_bind(fd, &tbind, &tbind) == -1)
		{
		t_error("Bind to server socket failed, program aborted");
		return;
		}

/*
 *   Display the local network and node returned from the t_bind and
 *   the well known server socket.
 */
	printf("TLI-SPX based timeserver started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*
 *                 Main Processing Loop
 *
 *    Since we opened in non-blocking mode we will look for connection,
 *    then sleep until one actually arrives. If a key is pressed, we abort.
 *    non blocked mode so that we may abort from the keyboard.
 */
	connection_open = FALSE;
	while ( !kbhit() )       /* allow user to abort program */
		{
	/*   Setup area to receive connection information */

		tcall.addr.buf     = (char *)&clientaddress;
		tcall.addr.maxlen  = sizeof(IPXAddress);
		tcall.udata.buf    = (char *)spx_data;
		tcall.udata.maxlen = 512;
		tcall.opt.maxlen   = 0;	  /* dont care to see them */

	/*
	 *   Loop, When a connection is received t_listen will return 0,
	 *   elsewise it should return -1 and t_errno will be set
	 *   to TNODATA ( no data available yet )
	 */
		connection_received = FALSE;		 
		while ( ! connection_received )
			{
			sleep(1); 
			if (kbhit())    
				{
				printf("Program canceled from keyboard\n");
				return;
				}
			if ( t_listen(fd, &tcall ) == 0)
				connection_received = TRUE;
			else
				{
				if (t_errno != TNODATA)
					{
					t_error("Fatal error during t_listen");
					return;
					}
				}
			}  /* end while ! connection_received */

		printf("Connection indication received on time server socket from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",clientaddress.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",clientaddress.node[i]);
		printf("\n\n");

	/*
	 *   Now we must accept the connection before it is truly open.
	 *   We use the same t_call structure from the indication as it
	 *   is set up with the connection information. 
	 *   We accept the connection on the same endpoint (fd) thereby
	 *   disabling anyone else from connecting to us until the
	 *   current connection is released.
	 */
		tcall.addr.len  = 0;  /* Toss any data received in connect indication */
		if (t_accept( fd, fd, &tcall) != 0 )
			{
			t_error("Connection acceptance failed, program aborted");
			return;
			}
		printf("Connection accepted\n");
	 	connection_open = TRUE;

	/*
	 *   We have a connection.   Process any and all received data
	 *   packets received from this connection.
	 */
		while ( connection_open )  /* process all incoming packets  */
			{
		/*
		 *   Wait for some data to come. Since we opened as
		 *   non blocking, t_rcv may fail for a while returning
		 *   TNODATA until data is received.  We may also encounter
		 *   a disconnect via a TLOOK error so handle that here also.
		 */
			data_received = FALSE;
			while ( ! data_received )  /* wait for some data to come */
				{
				sleep(1); 
				if (kbhit())    
					{
					printf("Program canceled from keyboard\n");
					return;
					}

				if ( t_rcv(fd, spx_data, 512, &recv_flags) != -1 )
					data_received = TRUE;
				else
					{ /* This failure may be 1. T_LOOK, A. Disconnect received
				 		*							            B. Other Fatal error
						*	                   2. TNODATA - No data yet, thats OK
				 		*							 3. Fatal Error
				 		*/
					if (t_errno == TLOOK)
						{
						if(t_look(fd) == T_DISCONNECT)
							{ /*  Process the disconnect */
							if (t_rcvdis(fd,NULL) == -1)
								{
								t_error("T_rcvdis fails, program aborted");
								return;
								}
							connection_open = FALSE;
							data_received   = TRUE; /* set to exit while gracefully */
							}
						else
							{
							t_error("T_look error during t_rcv, program aborted");
							return;
							}
						}
					else /* not a T_LOOK, better be no data error */
						if (t_errno != TNODATA)
							{
							t_error("Fatal error during t_rcv");
							return;
							}
					} /* end checking t_rcv failure */

				}  /* end while ! data_received */
		/*
		 *   After that last while we either have real data to process
		 *   or the connection was closed out.  If there is still
		 *   a connection open, it was data, process it.
		 */
			if (connection_open)
				{
				if (spx_data[0] != TIME_REQUEST)
					{
					printf("\nNon TIME REQUEST received on socket, ignored\n\n");
					break;
					}
				else
				/*   This seems to be a valid request, build a time packet and
				 *   send it back to the client.
				 */
					{
					printf("\nValid TIME REQUEST received, processing now.\n");
	
				/*
				 *   Get the time into the data area at the last possible moment
				 *   and then send the time packet on its merry way.
				 */	
					spx_data[0] = TIME_REPLY;      /* time reply operation   */
					time_of_day = time(NULL);      /* get latest time        */
					time_zone = getenv("TZ");		 /* and time zone          */
					memcpy(&spx_data[1], &time_of_day, sizeof(time_t));
					strcpy(&spx_data[1+sizeof(time_t)],time_zone);

				/*   Compute the length of send packet and let it fly */

					send_size = 1 	                 /* time reply       */
									+ sizeof(time_t)	  /* time             */
									+ strlen(time_zone) /* time zone string */
									+ 1;                /* string null      */
					send_flags = 0;

					if (t_snd(fd, spx_data, send_size, send_flags) == -1)
						{
						t_error("Fatal error, time reply send unsuccessful");
						return;
						}

					printf("TIME REPLY sent successfully.\n\nPress any key to abort program\n\n");

					} /* end valid TIME REQUEST */

				} /* end processing of data packet */

			} /* end while connection_open */

		} /* end while !kbhit() */

	return;
}  /* end main() */
